//
package genetic;

import java.util.*;
import java.io.*;

import kbm2l.*;
import tables.*;

class Estadistica {

  // Datos miembro

   // Poblacion sobre la que se obtienen las medidas

   Poblacion poblacion;

   // Para indicar el numero de muestras recogidas

   int muestras;

   // Para recoger los valores de fitness de los mejores individuos

   Vector mejores;

   // Para recoger los valores de fitness de los peores individuos

   Vector peores;

   // Para recoger los valores medios de fitness de cada generacion

   Vector medias;

   // Vectores para controlar la variabilidad

   Vector variabilidad_objetivo;
   Vector variabilidad_objetivo_repeticiones;

   // Para recoger el valor medio de fitness de todas las generaciones

   double valor_medio;

   // Para recoger el grado de repeticion de los genes de una poblacion. Cada
   // posicion alude a uno de los genes posibles. Este vector indica cuantas
   // veces aparece el mismo valor (gen) en la misma posicion de cada uno de los
   // individuos. Evidentemente se almacena el valor maximo

   Vector repeticiones_una_generacion;

   // Para controlar las repeticiones ocurridas a lo largo de todas las
   // generaciones

   Vector repeticiones_todas_generaciones;

  // Metodos miembro

   // Constructor

   Estadistica(Poblacion pob) {
   
      // Asignacion de la poblacion

      poblacion=pob;
 
      // Creacion del vector mejores

      mejores=new Vector();

      // Creacion del vector peores

      peores=new Vector();

      // Creacion del vector de medias de media

      medias=new Vector();

      // Creacion del vector de repeticiones de una generacion

      repeticiones_una_generacion=new Vector();

      // Creacion del vector de repeticiones de todas las generaciones

      repeticiones_todas_generaciones=new Vector();

      // Creacion del vector de variabilidad de la funcion objetivo

      variabilidad_objetivo=new Vector();

      // Creacion del vector de repeticiones en los valores del objetivo

      variabilidad_objetivo_repeticiones=new Vector();

      // Inicializacion del valor medio

      valor_medio=0;

      // Inicializacio del numero de muestras

      muestras=0;
   }

   // Metodo para agregar valores de fitness e incrementar el numero
   // de muestras

   public void agregar_muestras(double mejor, double peor, double media) {
//System.out.println("Agregando muestra (" +muestras+ ")" + mejor + ".." + peor + ".." + media);
      mejores.addElement(new Double(mejor));
      peores.addElement(new Double(peor));
      medias.addElement(new Double(media));
      muestras++;
   }

   // Metodo para escribir todos estos valores en un archivo

   public void mostrar_muestras() throws IOException{
    FileWriter f;
    PrintWriter p;
    int i;

      f=new FileWriter(dato30.workspace + File.separatorChar + "medidas.txt");
      p=new PrintWriter(f);

      p.println("Mejor  Peor   Media");
      for(i=0; i < muestras; i++) {
        p.print(((Double)mejores.get(i)).doubleValue());
        p.print("  "+((Double)peores.get(i)).doubleValue());
        p.print("  "+((Double)medias.get(i)).doubleValue());
        p.println("");
      }

      f.close();
   }

   // Metodo para escribir todos los valores de variabilidad en un
   // archivo 

   public void mostrar_variabilidad() throws IOException{
    FileWriter f;
    PrintWriter p;
    int i;

      f=new FileWriter(dato30.workspace + File.separatorChar + "variabilidad.txt");
      p=new PrintWriter(f);

      p.println("Valor  Repeticiones");
      for(i=0; i < variabilidad_objetivo.size(); i++) {
        p.print(((Double)variabilidad_objetivo.get(i)).doubleValue());
        p.print("  "+((Integer)variabilidad_objetivo_repeticiones.get(i)).intValue());
        p.println("");
      }

      f.close();
   }
         

   // Metodo para determinar el grado de repeticiones en los genes

   public void determinar_repeticiones() {
     int i,j,k;
     Individuo individuo;
     Individuo otro_individuo;
     Object gen, otro_gen;
     int repeticiones;
     int num_genes;
     double porc;

      // Se inicializa el vector de repeticiones

      inicializar_vectores_repeticiones();

      // Recorrido sobre todo el conjunto de individuos de la poblacion

      for(i=0; i < poblacion.acceder_numero_individuos(); i++) {

         // Accedemos al individuo en la posicion i

         individuo=poblacion.acceder_individuo(i);

         // Accedemos al numero de genes

         num_genes=individuo.acceder_numero_genes();

         // Recorrido sobre sus genes

         for(j=0; j < num_genes; j++) {
           // Para cada uno de los genes miramos si aparece repetido,
           // cuantas veces, en cada uno de los demas individuos

           gen=individuo.acceder_gen(j);

           // Recorrido sobre el resto de individuos

           repeticiones=1;
           for(k=0; k < poblacion.acceder_numero_individuos(); k++) {
               // Se considera si no es el mismo

               if (i != k) {
                   // Ver si gen aparece repetido en los demas individuos

                   otro_individuo=poblacion.acceder_individuo(k);

                   // Acceder al gen de este nuevo individuo, en la posicion j

                   otro_gen=otro_individuo.acceder_gen(j);

                   // Si son iguales se incrementa el contador

                   if (gen.equals(otro_gen) == true)
                         repeticiones++;
               }
           }

           // Al terminar el recorrido sobre este gen se actualiza el
           // vector de repeticiones

        if (repeticiones > ((Double)repeticiones_una_generacion.get(j)).doubleValue())
             repeticiones_una_generacion.set(j,new Double(repeticiones));
         }
      }

      // Se pasa a porcentajes el vector de repeticiones

      for(i=0; i < repeticiones_una_generacion.size(); i++) {
        porc=((Double)repeticiones_una_generacion.get(i)).doubleValue()*100/poblacion.acceder_numero_individuos();

        // Se almacena el procentaje

        repeticiones_una_generacion.set(i,new Double(porc));
      }

      // Se actualiza tambien el vector de repeticiones de todas las generaciones
      // Si es la primera generacion, se copian y ya. En caso contrario se han de
      // actualizar los valores 

      if (muestras == 0)
          repeticiones_todas_generaciones=(Vector)repeticiones_una_generacion.clone();
       else {
          // Hay que recalcular los porcentajes
       }
   } 

   // Metodo para inicializar los vectores de repeticiones

   private void inicializar_vectores_repeticiones() {
      int num_gen;
      Individuo indi;
      int i;

        // Se accede a un individuo para determinar cuantos genes tiene

        indi=poblacion.acceder_individuo(0);

        // Se accede al numero de genes

        num_gen=indi.acceder_numero_genes();

        for(i=0; i < num_gen; i++) {
           repeticiones_una_generacion.addElement(new Double(0));
           repeticiones_todas_generaciones.addElement(new Double(0));
        }
   }

   // Metodo para controlar la variabilidad de la funcion objetivo

   void controlar_variabilidad(){
     Individuo indi;
     double valor;
     Object objeto;
     int pos,repeticiones;
     int i;

      for(i=0; i < poblacion.acceder_numero_individuos(); i++){
        indi=poblacion.acceder_individuo(i);
        valor=indi.acceder_objetivo(); 

        // Vemos si este valor ya esta apuntado en el vector
        // de valores posibles de la funcion objetivo

        objeto=new Double(valor);
        if ((pos=variabilidad_objetivo.indexOf(objeto)) == -1){
          // El valor de objetivo es nuevo y no estaba apuntado

          variabilidad_objetivo.addElement(objeto);
          variabilidad_objetivo_repeticiones.addElement(new Integer(1));
        }
        else{
          // Se trata de un valor que ya esta

          objeto=variabilidad_objetivo_repeticiones.elementAt(pos);
          
          // Recuperamos el valor
 
          repeticiones=((Integer)objeto).intValue();
          repeticiones++;
          objeto=new Integer(repeticiones);
          variabilidad_objetivo_repeticiones.setElementAt(objeto,pos);
        }
      }
   }
}
