//
package genetic;

import java.util.*;
import java.io.*;

import kbm2l.*;

class EvolAev extends Evolucion {

  // Datos miembro

   // Individuo bajo consideracion

   Individuo individuo;

   // Para indicar el numero de cambios admitidos

   int ventana;

   // Para almacenar el maximo tamao de la ventana

   int maximaVentana;

   // Para controlar el numero de individuos que hemos
   // considerado

   private double considerados;

   // Para controlar el numero de ciclos sin mejora

   double ciclos_sin_mejora;

   // Para estadisticas

   EstadisticaAev estadistica;

  // Metodos miembro

   // Constructor de la clase

   EvolAev(Individuo indi, String sentido, int lim_cic_sin_mej){

     // Se asigna el sentido de la busqueda

     maximizacion=sentido.equalsIgnoreCase("max");

     // Se asigna el numero de ciclos sin mejora admitidos

     limite_ciclos_sin_mejora=lim_cic_sin_mej;

     // Se asigna el individuo considerado
   
     individuo=indi; 

     // La ventana comienza valiendo 1

     ventana=1;

     // Se asigna el maximo tamao de ventana

     maximaVentana=indi.acceder_numero_genes();
     frame30.kbm2lBrowser.append("MAXIMA VENTANA: "+maximaVentana+"\n");

     // Se inicializa considerados a 0

     considerados=0;

     // Se inicializa el numero de ciclos que llevamos sin
     // obtener mejora

     ciclos_sin_mejora=0;

     // Se crea un objeto de la clase EstadisticaAev

     estadistica=new EstadisticaAev();
   }

   // Para la evolucion de la poblacion 

   public void evolucionar() {
     boolean fin=false;

     // Calculamos el fitness del individuo 

     individuo.calcular_valor_objetivo();
     estadistica.agregar_muestras(individuo.acceder_objetivo());

     // Bucle principal de actuacion

     for (;ventana <= maximaVentana && fin == false; ventana++){
       // Se trata este tamao de ventana
       frame30.kbm2lBrowser.append("\n\nTRATAMIENTO DE VENTANA: "+ventana+"\n");
       fin=tratar_ventana();
     }

     // Se muestra el individuo obtenido

     individuo.visualizar();
   }

   // Metodo privado para realizar cambios sobre una determinada
   // ventana

   private boolean tratar_ventana(){
     double pruebas;
     Individuo indi_nuevo;

     // Se determina si se ha superado el numero
     // de vueltas al algoritmo sin mejora

     if (ciclos_sin_mejora > limite_ciclos_sin_mejora){
       // Parar: limite de funcionamiento

       return true;
     }

     pruebas=determinar_numero_pruebas();

     // Bucle de generacion de nuevos individuos, a partir de
     // mutaciones sobre el individuo considerado

     for(;pruebas > 0; pruebas--){

       indi_nuevo=new Individuo(individuo);

       // Este nuevo individuo se muta de acuerdo a la ventana

       indi_nuevo.mutarAev(ventana);
       frame30.kbm2lSystem.estatus("Objetivo previo: "+individuo.acceder_objetivo());
       frame30.kbm2lSystem.Xestatus("   alternativo: "+indi_nuevo.acceder_objetivo());

       // Se comprueba si el valor del nuevo individuo mejora
       // al del anterior
    
       if (hay_mejora(indi_nuevo)){
         ciclos_sin_mejora=0;
   
         // Continuamos trabajando ahora con este individuo

         individuo=indi_nuevo;

         // Se apunta el valor de fitness

         estadistica.agregar_muestras(individuo.acceder_objetivo());
         individuo.visualizar();
       }
        else
         ciclos_sin_mejora++;
     }

     return false;
   }

   // Metodo para determinar si hay mejora

  public boolean hay_mejora(Individuo indi_nuevo) {
    double previo;
    double actual;
    int muestras=estadistica.muestras;
    boolean mejora;

      // Se recuperan los valores previos

      previo=individuo.acceder_objetivo();
      actual=indi_nuevo.acceder_objetivo();
      mejora=false;

      if (maximizacion) {
         if(actual > previo)
            mejora=true;
      }
      else {
         if (actual < previo)
            mejora=true;
      }

      // Se devuelve el valor de mejora

      return(mejora);
  }

  private double determinar_numero_pruebas(){
     double posibles_cambios;
     double pruebas;
     double resto;
     double porcentaje;
     int potencia;

     // Calculamos el numero de posibles cambios, para proceder
     // a aumentar la ventana cuando corresponda

     posibles_cambios=Math.pow(maximaVentana-1,ventana);
     frame30.kbm2lBrowser.append("Posibles cambios: "+posibles_cambios+"\n");

     resto=posibles_cambios;
     for(potencia=1; ;  potencia++){
        resto=resto/10;
        frame30.kbm2lBrowser.append("Potencia = "+potencia+"\n");

        if (resto < 10)
          break;
     }

     frame30.kbm2lBrowser.append("Potencias de 10: "+potencia+"\n");

     // Determinamos el porcentaje de cambios que se van a probar

     porcentaje=(maximaVentana/potencia)*100;
     frame30.kbm2lBrowser.append("Porcentaje : "+porcentaje+"\n");

     // Sobre este porcentaje calculamos cuantas pruebas haremos

     pruebas=(porcentaje*posibles_cambios)/100;
     frame30.kbm2lBrowser.append("Pruebas: "+pruebas+"\n");

     // Redondeamos, para que sea entero

     pruebas=Math.floor(pruebas);

     // Se devuelve el numero de pruebas

     return pruebas;
  }
}
