//
package genetic;

import java.util.*;
import java.io.*;

import kbm2l.*;

class EvolNormal extends Evolucion {

  // Datos miembro

  // Metodos miembro

  EvolNormal(Poblacion pob, String sentido, double prob_sel, 
             double prob_mut, double prob_des, int lim_cic, 
             int lim_cic_sin_mej,
             String op_cruce) {

   // Se asignan los datos miembro correspondientes

   maximizacion=sentido.equalsIgnoreCase("max");

   // Se asigna la probabilidad de seleccion

   probabilidad_seleccion=prob_sel;

   // Se asigna la probabilidad de mutacion

   probabilidad_mutacion=prob_mut;

   // Se asigna la probabilidad de descarte

   probabilidad_descarte=prob_des;
  
   // Se asigna el numero maximo de generaciones

   limite_ciclos=lim_cic;

   // Se asigna el numero maximo de ciclos sin mejora

   limite_ciclos_sin_mejora=lim_cic_sin_mej;

   // Se asigna la poblacion

   asignar_poblacion(pob);

   // Se inicializa a 0 el numero de ciclos

   ciclos=0;

   // Se inicializa a 0 el contador de ciclos sin mejora

   ciclos_sin_mejora=0;

   // Se inicializa el operador de cruce

   operador_cruce=op_cruce;

   // Se crea un objeto de la clase Estadistica

   estadistica=new Estadistica(poblacion);
  }

  public void evolucionar() {
   int i,descartados,a_descartar;
   int num_individuos_inicial;
   Vector seleccionados_cruce=new Vector();
   Vector seleccionados_descarte=new Vector();
   Individuo padre,madre,indi;
   double prob;

    // Se apuntan los resultados de la poblacion inicial
    poblacion.actualizar_mejor_peor();
    apuntar_resultados();
     
    // Bucle principal de funcionamiento
    for(ciclos=0; ciclos < limite_ciclos && 
                  ciclos_sin_mejora < limite_ciclos_sin_mejora; ciclos++) {


      // Se seleccionan los individuos para seleccion. Para ello se
      // crea un selector para el cruce
      selector_cruce=new LinearRankSelector(maximizacion,
                              probabilidad_seleccion,poblacion);

      // Mediante este selector hemos de seleccionar numero_individuos/2
      num_individuos_inicial=poblacion.acceder_numero_individuos();
      for(i=0; i < num_individuos_inicial/2; i++) {

         // Cada uno de los individuos seleccionados se introduce
         // en el vector de seleccionados
         seleccionados_cruce.add((Object)selector_cruce.seleccionar());
      }//for

      // Se baraja esta lista de individuos seleccionados
      General.barajar(seleccionados_cruce);

      // Una vez seleccionados y barajados, se van cruzando
      for(i=0; i < seleccionados_cruce.size()/2; i++) {
          padre=(Individuo)seleccionados_cruce.get(i);
          madre=(Individuo)seleccionados_cruce.get(i+1);

          if (operador_cruce.equalsIgnoreCase("pozo") == true) 
              poblacion.cruzar_del_pozo(padre,madre);
           else
              poblacion.cruzar_punto_individuos(padre,madre);
      }

      // Forzamos la deseleccion de todos los individuos

      poblacion.deseleccionar();

      // Una vez hecho el cruce, se produce la mutacion sobre
      // todos los individuos de la poblacion

      for(i=0; i < poblacion.acceder_numero_individuos(); i++) {

         // Se genera un valor aleatorio, y si es menor que la
         // probabilidad de mutacion, se muta

         indi=(Individuo)poblacion.acceder_individuo(i);
         prob=General.generar_prob_uniforme();

         if (prob <= probabilidad_mutacion) 
            indi.mutar(1);
      }//for

      // Llegados a esta punto hemos de actualizar las medidas

      poblacion.actualizar_mejor_peor();

      // Una vez hecha la mutacion, se seleccionan los individuos
      // a descartar: han de ser numero_individuos/2

      selector_descarte=new LinearRankSelector(!maximizacion,
                                 probabilidad_descarte,poblacion);

      a_descartar=poblacion.acceder_numero_individuos()-num_individuos_inicial;
      for(descartados=0; descartados < a_descartar;descartados++) {
          indi=selector_descarte.seleccionar();
          seleccionados_descarte.add((Object)indi);

          // Si es el de mayor valor objetivo, que cante

          if (maximizacion == true) {
            if (indi.acceder_objetivo() == 
                  poblacion.acceder_mayor_valor_objetivo()) {
               frame30.kbm2lBrowser.append("||||||||||||||||||||||\n");
            }
          }
      }

      // Una vez seleccionados, se eliminan de la poblacion

      for(i=0; i < seleccionados_descarte.size(); i++) {
          indi=(Individuo)seleccionados_descarte.get(i);
          poblacion.eliminar_individuo(indi);
      }

      // Llegados a esta punto hemos de actualizar las medidas

      poblacion.actualizar_mejor_peor();

      // Se apuntan los resultados

      apuntar_resultados();

      // Se determina si hay o no mejora
      if (hay_mejora())
          ciclos_sin_mejora=0;
       else ciclos_sin_mejora++;

      // Se limpian los vectores seleccionados_cruce y 
      // seleccionados_descarte

      seleccionados_descarte.removeAllElements();
      seleccionados_cruce.removeAllElements();

      // Se actualiza la edad de los individuos

      poblacion.actualizar_edad_individuos();

      // Se muestra informacion

      try{
         mostrar_informacion();
      }catch(IOException e){};
    }
  }//evolucionar

  // Para apuntar los resultados de una generacion
  public void apuntar_resultados() {

    double menor;
    double mayor;
    double media;

      // Se extrae el valor menor de la funcion objetivo

      menor=poblacion.acceder_menor_valor_objetivo();
      mayor=poblacion.acceder_mayor_valor_objetivo();

      media=(mayor+menor)/2;

      // Se apuntan en la estadistica

      if (maximizacion) 
         estadistica.agregar_muestras(mayor,menor,media);
       else
         estadistica.agregar_muestras(menor,mayor,media);

      // Tambien se apuntan los valores objetivo de todos
      // los individuos, de forma que sepamos la variabilidad
      // de la funcion objetivo: cuntas veces aparece cada
      // valor

      estadistica.controlar_variabilidad();
  }

  public boolean hay_mejora() {
    double previo;
    double actual;
    int muestras=estadistica.muestras;
    boolean mejora;

      // Se recuperan los valores previos

      previo=((Double)estadistica.mejores.get(muestras-2)).doubleValue();
      actual=((Double)estadistica.mejores.get(muestras-1)).doubleValue();

      mejora=false;

      if (maximizacion) {
         if(actual > previo)
            mejora=true;
      }
      else {
         if (actual < previo)
            mejora=true;
      }

      // Se devuelve el valor de mejora

      return(mejora);
  }

  private void mostrar_informacion() throws IOException {
   Individuo indi;
   double actual;
   int muestras=estadistica.muestras;

    estadistica.mostrar_muestras();
    estadistica.mostrar_variabilidad();

    actual=((Double)estadistica.mejores.get(muestras-1)).doubleValue();

    frame30.kbm2lBrowser.append(" .................. " + ciclos + " ................."+"\n");
    frame30.kbm2lBrowser.append(" MEJOR: " + actual + "   ");
    frame30.kbm2lBrowser.append(" Ciclos sin mejora : " + ciclos_sin_mejora+"\n");

    if (maximizacion)
      indi=poblacion.acceder_individuo_mayor_objetivo();
     else
      indi=poblacion.acceder_individuo_menor_objetivo();

    // Se muestra el individuo

    indi.visualizar();
  }
}
