//
package genetic;

import java.io.*; 
import java.util.*;
import java.lang.*;

final class General {

   static Random generador;

   // BLOQUE STATIC: se ejecuta la primera vez que se
   // carga la clase

   static {
       generador=new Random();
   }

   // Todos los metodos son estaticos

   public static void inicializar_generador() {
   
     Date fecha=new Date();
     long semilla;

      // Esta fecha se usa para obtener el numero de milisegundos
      // correspondiente

      semilla=fecha.getTime();

      // La semilla se usa para inicializar el generador

      generador.setSeed(semilla);
   } 

   // Metodo para generar un numero entero, segun una
   // distribucion uniforme, entre min y max, ambos
   // inclusive

   public static int generar_ent_uniforme(int min, int max) {
   
     int generado;

      // Se genera un valor

      generado=Math.abs(generador.nextInt());

      // Se lleva a los limites requeridos

      generado=(generado%(max-min+1))+min;

      // Se devuelve el valor generado

      return(generado);
   }

   public static double generar_prob_uniforme() {

      // Se genera el valor aleatorio

      return(generador.nextDouble());
    }

   public static double generar_prob_normal() {

      // Se genera el valor aleatorio

      return(generador.nextGaussian());
   }

   // Para hacer algo similar al lanzamiento de monedas,
   // devolviendo 0 o 1

   public static int lanzar_moneda() {
     int generado;

      // Se genera un valor

      generado=Math.abs(generador.nextInt());

      // Se devuelve el resto de dividir por 2
      return(generado%2);
   }

   // Metodo para generar numero aleatorio, especialmente
   // pensado para algoritmos de entorno variable. Este
   // metodo ha de generar un numero entero aleatorio, en
   // un rando determinado, pero evitando que sea un valor
   // determinado

   public static int generar_valor_aev(int min, int max, int no) {
     int mitad;
     int generado;

      inicializar_generador();

      // Si el valor "no" esta en medio, se decide si generar
      // valor en una parte o en otra

      if (no > min && no < max) {

          // De ser asi, vemos con que mitad nos quedamos:
          // O- izquierda, 1- derecha

          mitad=lanzar_moneda();

          // Ahora hemos de generar un numero aleatorio
          // de alguna de las mitades

          if (mitad == 0) {
              generado=generar_ent_uniforme(min,no-1);
System.out.println("Generado en izqda, entre "+min+" y "+(no-1)+" = "+generado);
          }
          else {
              // Se genera de la parte derecha

              generado=generar_ent_uniforme(no+1,max);
System.out.println("Generado en decha, entre "+(no+1)+" y "+max+" = "+generado);
          }
      }
      else {
         // "no" se corresponde a uno de los extremos

        if (no == min) {
            generado=generar_ent_uniforme(no+1,max);
System.out.println("Generado en extremo izqdo, entre "+(no+1)+" y "+max+" = "+generado);
        }
        else {
            generado=generar_ent_uniforme(min,no-1);
System.out.println("Generado en extremo dcho, entre "+min+" y "+(no-1)+" = "+generado);
        }
      }

      // Se devuelve el valor generado

      return(generado);
   }

   public static int leer_entero(BufferedReader bufer) {
     Double valor=null;
     String cadena;

       try {
           cadena=bufer.readLine();
           valor=new Double(cadena);
       } 
       catch (IOException e) {
          // En caso de error se muestra el mensaje correspondiente

          System.out.println("Error en lectura de archivo: "+e);
          System.exit(-1);
       }

       // Se devuelve el valor recuperado

       return(valor.intValue());
   }

   public static double leer_double(BufferedReader bufer) {
     Double valor=null;
     String cadena;

       try {
           cadena=bufer.readLine();
           valor=new Double(cadena);
       } 
       catch (IOException e) {
          // En caso de error se muestra el mensaje correspondiente

          System.out.println("Error en lectura de archivo: "+e);
          System.exit(-1);
       }

       // Se devuelve el valor recuperado

       return(valor.doubleValue());
   }

   public static String leer_cadena(BufferedReader bufer) {
     String cadena=null;

       try {
           cadena=bufer.readLine();
       } 
       catch (IOException e) {
          // En caso de error se muestra el mensaje correspondiente

          System.out.println("Error en lectura de archivo: "+e);
          System.exit(-1);
       }

       // Se devuelve el valor recuperado

       return(cadena);
   }

   public static void barajar(Vector vec) {

    int longitud;
    int num_cambios;
    Object a_cambiar;
    int i;
    int pos_inicial=0;
    int pos_final=0;

     // El procedimiento de baraja selecciona al azar un
     // numero de cambios

     longitud=vec.size();

     num_cambios=generar_ent_uniforme(longitud,2*longitud);

     // Bucle de realizacion de los cambios

     for(i=0; i < num_cambios; i++) {
        // Se selecciona la posicion_inicial

        pos_inicial=generar_ent_uniforme(0,longitud-1);

        do {
          pos_final=generar_ent_uniforme(0,longitud-1);
        } while(pos_inicial == pos_final);
     }

     a_cambiar=vec.get(pos_final);
     vec.set(pos_final,vec.get(pos_inicial));
     vec.set(pos_inicial,a_cambiar);
   }

   public static Vector buscar_en_vector(Vector vector, Object objeto) {
     Vector indices=new Vector();
     int pos_inicio=0;
     int pos_localizado;

       // Se busca en el vcetor pasado como parametro

       do {
         pos_localizado=vector.indexOf(objeto,pos_inicio);

         // Si aparece, se inserta en el vector de indices

         if (pos_localizado != -1) {
             indices.add(new Integer(pos_localizado));
             pos_inicio=pos_localizado+1;
         }
       } while(pos_inicio < vector.size() && pos_localizado != -1);

       // Se devuelve el vector de indices

       return(indices);
   }

   public static void main(String args[]) {

     int i;

      inicializar_generador();
      for(i=0; i < 100; i++) {
        System.out.println("Valor "+i+" = "+ generar_ent_uniforme(56,67));
      }

      i=generar_valor_aev(3,7,3);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,4);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,5);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,3);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,4);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,5);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,3);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,4);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,5);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,3);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,4);
      System.out.println("Valor generado = "+i);
      i=generar_valor_aev(3,7,5);
      System.out.println("Valor generado = "+i);
   }
}

      
