//
package genetic;

//
import java.util.*;
import java.io.*;

//
import kbm2l.*;
import kernel.*;
import target.*;
import tables.*;

public  class  Genetico  implements  Runnable {
  // Datos miembro

   // Poblacion sobre la que se trabaja 

   Poblacion poblacion;

   // Parametro para definir el numero de individuos
   // de la poblacion. Este valor se lee de un archivo
   // de configuracion: config.dat 

   int numero_individuos=0;

   // En el mismo archivo se configuran la probabilidad
   // de mutacion

   double prob_mutacion=0;

   // Probabilidad de seleccion

   double prob_seleccion=0;

   // Probabilidad de descarte

   double prob_descarte=0;

   // Operador de seleccion

   String operador_seleccion;

   // Operador de mutacion

   String operador_mutacion;

   // Sentido de la busqueda: maximizacion o minimizacion

   String objetivo;

   // Maximo numero de ciclos permitidos

   int limite_ciclos;

   // Maximo numero de generaciones sin mejora

   int limite_ciclos_sin_mejora;

   // Nombre del operador de cruce usado

   String operador_cruce;

  // Metodos miembro

public  Genetico ( lista30 l30) {

}

  Genetico (fitnessMgo fitness) {

    // Lectura de los parametros de configuracion

    lectura_parametros();

    // Se la poblacion

    poblacion=new Poblacion(fitness,numero_individuos);
  }

  // Metodo para la lectura de los parametros de configuracion.
  // Los valores definidos en el son:
  //  a) Numero de individuos de la poblacion 
  //  b) Probabilidad de mutacion
  //  c) Probabilidad de seleccion
  //  d) Probabilidad de descarte
  //  e) Operador de seleccion
  //  f) Operador de mutacion
  //  g) Sentido de la busqueda
  //  h) Numero maximo de generaciones de funcionamiento
  //  i) Numero maximo de generaciones sin mejora
  //  j) Operador de cruce

  private void lectura_parametros() {
    InputStreamReader lector;
    BufferedReader lector_bufer;
    FileInputStream file=null;
    Double valor;

    try {
       file=new FileInputStream(dato30.workspace + File.separatorChar + "config.dat");
    }
    catch(FileNotFoundException e) {
       System.out.println("No existe el archivo config.dat");
       System.exit(0);
    }

    lector=new InputStreamReader(file);
    lector_bufer=new BufferedReader(lector);

    // En caso de existir leemos los siguientes valores:

    // a) Numero de individuos de la poblacion, en la primera
    //    linea

    numero_individuos=General.leer_entero(lector_bufer);

    // En la siguiente linea ha de aparecer la probabilidad de mutacion

    prob_mutacion=General.leer_double(lector_bufer);

    // En la siguiente la probabilidad de seleccion

    prob_seleccion=General.leer_double(lector_bufer);

    // En la siguiente la probabilidad de descarte

    prob_descarte=General.leer_double(lector_bufer);

    // En la proxima el operador de seleccion

    operador_seleccion=General.leer_cadena(lector_bufer);

    // A continuacion el operador de mutacion

    operador_mutacion=General.leer_cadena(lector_bufer);

    // Finalmente si es un problema de maximizacion o minimizacion

    objetivo=General.leer_cadena(lector_bufer);

    // Se lee el numero maximo de generaciones

    limite_ciclos=General.leer_entero(lector_bufer);

    // Se lee el maximo numero de iteraciones sin mejora

    limite_ciclos_sin_mejora=General.leer_entero(lector_bufer);

    // Se lee el nombre del operdor de cruce

    operador_cruce=General.leer_cadena(lector_bufer);

    frame30.kbm2lBrowser.append("***********************************************\n");
    frame30.kbm2lBrowser.append("Numero de individuos:      "+numero_individuos+"\n");
    frame30.kbm2lBrowser.append("Probabilidad de mutacion:  "+prob_mutacion+"\n");
    frame30.kbm2lBrowser.append("Probabilidad de seleccion: "+prob_seleccion+"\n");
    frame30.kbm2lBrowser.append("Probabilidad de descarte:  "+prob_descarte+"\n");
    frame30.kbm2lBrowser.append("Operador de seleccion:     "+operador_seleccion+"\n");
    frame30.kbm2lBrowser.append("Operador de mutacion:      "+operador_mutacion+"\n");
    frame30.kbm2lBrowser.append("Sentido de la busqueda:    "+objetivo+"\n");
    frame30.kbm2lBrowser.append("Numero maximo de iteraciones: "+limite_ciclos+"\n");
    frame30.kbm2lBrowser.append("Numero maximo de iter. sin mejora: "+limite_ciclos_sin_mejora+"\n");
    frame30.kbm2lBrowser.append("Operador de cruce: "+operador_cruce+"\n");
    frame30.kbm2lBrowser.append("***********************************************\n");
  } 

/**
* entry-point
**/
  public static lista30 genman( ) {
    Individuo peor;
    Individuo mejor;
    Evolucion evol;
 
    // Se comprueba que se introduce el nombre del ejemplo
    // sobre el que trabajar

//    if (args.length < 1){
//      System.out.println("Especificar nombre base ");
//      System.exit(-1);
//    }

    // Creo el objeto de la clase fitness asociado a esta base
    fitnessMgo fitness=new fitnessMgo(frame30.activel30.Name());

    // Creo el nuevo genetico
    Genetico gene=new Genetico(fitness);

    // Se crea un objeto de la clase EvolNormal
    evol=new EvolNormal(gene.poblacion, gene.objetivo, 
                        gene.prob_seleccion,
                        gene.prob_mutacion,gene.prob_descarte,
                        gene.limite_ciclos,gene.limite_ciclos_sin_mejora,
                        gene.operador_cruce);

    // Se hace evolucionar
    evol.evolucionar();

    // Se obtienen los mejores y peores individuos
    frame30.kbm2lBrowser.append("*** Genetico: Mejor y Peor ***\n");
    (mejor=gene.poblacion.extraer_individuo_menor_objetivo()).visualizar();
    (peor=gene.poblacion.extraer_individuo_mayor_objetivo()).visualizar();  
    
    lista30 l30 = frame30.activel30;
    if (( l30 = frame30.activel30.aTest( targetStd30.TargetG( frame30.activel30, 
                 target30.swap( frame30.activel30.Base, mejor.genes))))!= null)
      frame30.kbm2lBrowser.append( "mejor: " + l30.size()+"\n");  
      else frame30.kbm2lBrowser.append("no cumple aTest.\n");  
    
    return frame30.activel30;
  }//


  public static void main(String args[]) {
    Individuo peor;
    Individuo mejor;
    Evolucion evol;
 
    // Se comprueba que se introduce el nombre del ejemplo
    // sobre el que trabajar

    if (args.length < 1){
      System.out.println("Especificar nombre base ");
      System.exit(-1);
    }

    // Creo el objeto de la clase fitness asociado a esta base

    fitnessMgo fitness=new fitnessMgo(args[0]);

    // Creo el nuevo genetico

    Genetico gene=new Genetico(fitness);

    // Se crea un objeto de la clase EvolNormal

    evol=new EvolNormal(gene.poblacion, gene.objetivo, 
                        gene.prob_seleccion,
                        gene.prob_mutacion,gene.prob_descarte,
                        gene.limite_ciclos,gene.limite_ciclos_sin_mejora,
                        gene.operador_cruce);

    // Se hace evolucionar

    evol.evolucionar();

    // Se obtienen los mejores y peores individuos

    (mejor=gene.poblacion.extraer_individuo_menor_objetivo()).visualizar();
    (peor=gene.poblacion.extraer_individuo_mayor_objetivo()).visualizar();  
  }

/**
* run process
**/
public void run() {

  if ( frame30.activel30.dimension() > 1)
  try {
    long ts0 = (new Date()).getTime();

    frame30.activel30 = Genetico.genman();

    frame30.kbm2lTime.append("Sintesis Genetico:: " + (new Date().getTime() - ts0) + ".\n");      
  } catch (Exception e) {
    System.out.println( "ERROR  " + "run().");
    e.printStackTrace();
    System.exit( 0);
  }//catch

  return;
}//run

}//class Genetico

