//
package genetic;

import java.util.*;
import java.io.*;

import kbm2l.*;

public class Individuo {

  // Datos miembro

  public Vector genes;

  // Contador del numero de genes

  private int numero_genes;

  // Valor de fittness

  private double objetivo;

  // Valor para almacenar la probabilidad de
  // seleccion

  private double prob_seleccion;

  // Valor para almacenar la probabilidad de
  // mutacion

  private double prob_mutacion;

  // Valor para almacenar la probabilidad de
  // descarte

  private double prob_descarte;

  // Valor para almcenar la edad del individuo

  private int edad;

  // Para guardar la referencia al objeto de la clase
  // fitnessMgo

  private fitnessMgo fitness;

  // Para indicar si el individuo esta o no seleccionado

  private boolean seleccionado;

  // Metodos miembro

  Individuo (int longitud,fitnessMgo fitness) {
  int i;

  this.fitness=fitness;

  // Reserva de espacio para el vector de genes

  genes=new Vector(longitud);

  numero_genes=longitud;

  // Se asignan los valores del vector, inicialmente
  // desde 1 hasta longitud

  for(i=0; i < longitud; i++) {
      genes.add(i,new Integer(i));
  }

  // Se inicializa la edad a 0

  edad=0;

  // Se inicializa el objetivo a 0

  objetivo=0;

  // Se indica que no esta seleccionado

  seleccionado=false;
  }

  // Constructor que crea un nuevo individuo y copia el
  // vector de genes de otro individuo

  Individuo (Individuo original){
    int i;

    // Se reserva espacio para el nuevo array de genes

    genes=new Vector(original.numero_genes);

    // Se inicializa el numero de genes

    numero_genes=original.numero_genes;

    // Se copian los genes

    for(i=0; i < numero_genes; i++){
       genes.add(original.genes.elementAt(i));
    }

    // Se permite el uso del fitness

    fitness=original.fitness;

    // Se inicializa la edad

    edad=0;

    // Se inicializa el objetivo

    objetivo=0;

    // Se inidica que no esta seleccionado

    seleccionado=false;
  }

  // Metodo para copiar un individuo 

  Individuo copiar(){
    int i;
    Individuo indi=new Individuo(genes.size(),fitness);

    // Ahora se copian los genes

    for(i=0; i < genes.size(); i++){
      indi.genes.add(genes.elementAt(i));
    }

    // Se copia el valor de objetivo

    indi.objetivo=objetivo;

    // Se devuelve el nuevo individuo

    return indi;
  }

  // Metodo para seleccionar

  public void seleccionar(){
    seleccionado=true;
  }

  // Metodo para quitar seleccion

  public void deseleccionar(){
    seleccionado=false;
  }

  // Metodo para acceder al flag de seleccion

  public boolean acceder_seleccion(){
    return seleccionado;
  }

  // Metodo para asignar la probabilidad de seleccion

  void asignar_probabilidad_seleccion(double prob) {
     prob_seleccion=prob;
  }

  // Metodo para acceder a la probabilidad de seleccion

  double acceder_probabilidad_seleccion() {
     return(prob_seleccion);
  }

  // Metodo para acceder al gen de una determinada posicion

  public Object acceder_gen(int pos) {
     return(genes.get(pos));
  }

  // Metodo que devuelva la posicion en que se encuentra un gen

  public int acceder_posicion_de_gen(int valor) {
     return(genes.indexOf(new Integer(valor)));
  }

  // Metodo para asignar un gen a un valor 

  public void asignar_gen(int pos, Object gen) {
     genes.set(pos,gen);
  }

  // Metodo para acceder al numero de genes

  public int acceder_numero_genes() {

     // Se actualiza el numero de genes

     numero_genes=genes.size();
     return(numero_genes);
  }

  // Metodo para devolver el fitness del individuo

  public double acceder_objetivo() {
     return(objetivo);
  }

  // Metodo para incrementar la edad

  public void incrementar_edad() {
     edad++;
  }

  // Metodo para devolver la edad() {
    
  public int acceder_edad() {
     return(edad);
  }

  // Metodo para calcular el valor de objetivo

  public void calcular_valor_objetivo() {
     //objetivo=General.generar_prob_uniforme()*1000;
     objetivo=fitness.getFitness(genes);
  }   

  // Metodo para visualizar las caracteristicas del
  // individuo

  public void visualizar() {
    int i;

     frame30.kbm2lBrowser.append("\n******\n");

     // Se muestra la edad del individuo
     frame30.kbm2lBrowser.append("EDAD = "+edad+",  base:");

     // Se muestran los genes
     for(i=0; i < genes.size(); i++) {
       frame30.kbm2lBrowser.append(" "+genes.get(i)+"  ");
     }
     frame30.kbm2lBrowser.append("\n");

     // Se muestra el valor de objetivo
     frame30.kbm2lBrowser.append("Objetivo : "+objetivo+"\n");
     frame30.kbm2lBrowser.append(".......\n");

     // Se muestra la probabilidad de seleccion
     frame30.kbm2lBrowser.append("Prob. seleccion = "+prob_seleccion);

     // Se muestra si esta seleccionado o no
     frame30.kbm2lBrowser.append("   Seleccionado: "+ seleccionado+"\n");

   }

   // Metodo para hacer mutaciones, tantas como indique
   // el parametro num_mut

   public void mutar(int num_mut) {

     int i;
     int limite=genes.size()-1;
     int pos1,pos2;
     int valor;

      // Para cada una de las mutaciones seleccionamos
      // dos numeros entre 0 y limite. Estas
      // dos posiciones intercambian sus valores

      for(i=0; i < num_mut; i++) {
        // Seleccion aleatoria de la primera posicion

        pos1=General.generar_ent_uniforme(0,limite);

        do {
             pos2=General.generar_ent_uniforme(0,limite);
        } while (pos1 == pos2);

        // Intercambio de posiciones

        valor=((Integer)genes.get(pos1)).intValue();
        genes.set(pos1,genes.get(pos2));
        genes.set(pos2,new Integer(valor));
      }

      // Se debe calcular de nuevo su fitness

      calcular_valor_objetivo();
   }

   // Metodo de mutacion asociado a Algoritmos de entorno
   // variable

   public void mutarAev(int ventana){
    int limite=genes.size()-1;
    int posOrigen,posDestino;

     // El destino esta limitado a los genes de la izquierda,
     // tantos como se indiquen en ventana (de 0 a ventana-1)
     // Para los origenes no hay limitacion. No se permite
     // identico valor para ambas posiciones

     while(true){
       posOrigen=General.generar_ent_uniforme(0,limite);
       posDestino=General.generar_ent_uniforme(0,ventana-1);

       if (posOrigen != posDestino)
         break;
     }

     // Intercambio de posisicones

     int valor=((Integer)genes.get(posDestino)).intValue();
     genes.set(posDestino,genes.get(posOrigen));
     genes.set(posOrigen,new Integer(valor));

     // Se calcula de nuevo el valor del fitness

     calcular_valor_objetivo();
   }

   // Metodo para hacer mutaciones. Especialmente pensado para
   // su utilizacion en algoritmos de entorno variable. La idea
   // es permitir el cambio en cierto numero de genes
   // consecutivas, tantos como indique el primer valor pasado
   // como parametro, empezando por aquel asociado al indice
   // pasado como segundo argumento

   public void mutar(int num_cambios, int prim_pos) {
     int generado;
     int i;
     Object gen_inicial;

      // Para cada uno de los cambios generamos un numero
      // aleatorio entre prim_pos y numero_genes, teniendo
      // en cuenta que en cada iteracion hay un valor que
      // no se puede general: el correspondiente a la
      // posicion sobre la que se actua
      
      for(i=0; i < num_cambios; i++) {
         generado=General.generar_valor_aev(prim_pos,numero_genes,prim_pos+i);

         // Este valor generado me indica el cambio a hacer en el individuo

         gen_inicial=genes.get(prim_pos+i);

         // En esta posicion metemos el gen que esta en la posicion indicada
         // por generado

         genes.set(prim_pos+i,genes.get(generado));

         // En la posicion indicada por generado ponemos el valor existente
         // en el gen inicial

         genes.set(generado,gen_inicial);
      }

      // Se comprueba la validez del individuo asi generado

      if (comprobar() == false) {
         System.out.println("Error en aplicacion de mutar para AEV");
         System.exit(-10);
      }
   }


   // Metodo para comprobar la validez de un individuo. 
   // Deben estar todos los enteros entre 0 y el numero
   // de genes de los individuos

   public boolean comprobar() {

    int i;
    int limite=genes.size();
    
     // Recorrido de los numeros, entre 0 y limite-1

     for(i=0; i < limite; i++) {
       // Ver que este numero esta en la lista de genes

       if (genes.contains(new Integer(i)) == false)
           return false;
     }

     // De llegar aqui, todo ok

     return true;
   }

   // Procedimiento para reparar una secuencia de genes

   public void reparar() {
     int i,j;
     int limite=genes.size();
     int pos_a_reparar;
     Vector indices=new Vector();

      // En los genes han de estar los numeros entre 0 y limite-1,
      // sin repetir. Si alguno de ellos esta repetido, cada una
      // de las ocurrencias repetidas (no la primera) se sustituye, 
      // al azar, por uno de los genes que faltan
//System.out.println("................. REPARACION.................");
//visualizar();
//System.out.println(".............................................");
      for(i=0; i < limite; i++) {
          // Ver si el gen esta presente

          if (genes.contains(new Integer(i)) == true) {
//System.out.println("Considerando gen: "+i);

              // Ver si esta repetido

              buscar_repeticiones(i,indices);

              // Ver cuantas repeticiones aparecen, y se resuelven las
              // ultimas

              if(indices.size() > 1) {
//System.out.println("Tratando repeticiones del gen "+i);
//{
  //int j;
  //for(j=0; j < indices.size(); j++) {
     //System.out.println("Repetido en ...."+((Integer)indices.get(j)).intValue());
  //}
//}
                 // Hay repeticiones, y hemos de hacer reparaciones.
                 // La primera ocurrencia se respeta

                 // Para cada una de las restantes se busca un gen
                 // que falte y se repara

                 for(j=1; j < indices.size(); j++) {
                    pos_a_reparar=((Integer)indices.get(j)).intValue();
//System.out.println("  Reparando posicion....."+pos_a_reparar);

                    // Se llama al metodo de reparacion

                    reparar_posicion(pos_a_reparar);
                 }
              }

              // Limpiamos el array de indices

              indices.removeAllElements();
          }
      }

      // Al final comprobar si quedo reparado

      if (comprobar() == false) {
          System.out.println("Error en la reparacion");
          System.exit(-10);
      }
   }

   private void reparar_posicion(int pos) {
     Vector faltan=new Vector();
     int i;
     int seleccion;

       for(i=0; i < genes.size(); i++) {
           if (genes.contains(new Integer(i)) == false) {
               faltan.add(new Integer(i));
//System.out.println("Falta el gen......"+i);
           }
       }

       // Ahora se selecciona un valor aleatorio, si es
       // necesario

       seleccion=0;

       if (faltan.size() > 1) {
          seleccion=General.generar_ent_uniforme(0,faltan.size()-1);
//System.out.println("   Valor nuevo: "+((Integer)faltan.get(seleccion)).intValue());
       }

       // Con este valor se selecciona el gen a introducir en la posicion
       // pos

       genes.set(pos,faltan.get(seleccion));
   }

   private void buscar_repeticiones(int gen, Vector vec) {
     int pos;
     int encontrado;

      Integer elemento=new Integer(gen);

      // Buscamos todas las ocurrencias del valor

      pos=genes.indexOf(elemento,0);

      // Esta posicion se inserta en el vector

      vec.add(new Integer(pos));

      do {
        encontrado=genes.indexOf(elemento,pos+1);

        if (encontrado != -1) {
            pos=encontrado;
            vec.add(new Integer(pos));
        }
        else
          break;
      } while (pos < genes.size()); 
   }

   public static void main(String args[]) {
     Individuo ind;

      // Creacion del individuo

      ind = new Individuo(30,null);
      ind.visualizar();
      ind.mutar(1000);
      ind.visualizar();
      System.out.println("\nValido : "+ind.comprobar());
   }
}
    
  


