//
package genetic;

import java.util.*;

public class LinearRankSelector extends Seleccion {

  LinearRankSelector(boolean objetivo, double prob, Poblacion pob) {

   // Se asigna el sentido de la busqueda

   sentido=objetivo;

   // Se asigna el valor asociado a la probabilidad de seleccion

   probabilidad_seleccion=prob;

   // Se asigna tambien la poblacion

   asignar_poblacion(pob);

   // Se procede a hacer la ordenacion de la poblacion segun el
   // sentido de la busqueda

   poblacion.ordenar_individuos(sentido);

   // Se asigna a cada individuo de la poblacion un valor de
   // probabilidad 

   asignar_probabilidad_ranking();
  }

  // Implementacion del metodo seleccionar. En este caso, la seleccion
  // comienza ordenando los individuos de la poblacion, en funcion a su
  // valor de fitness, y luego asigna una probabilidad mediante una 
  // funcion de ranking lineal (p(1-p)^k). Posteriormente hay que 
  // convertir estos valores en una distribucion real de probabilidad
  // El valor de p se pasa como parametro en la construccion de este
  // selector

  Individuo seleccionar() {
    double prob;
    Individuo indi=null;
    boolean seleccionado=false;

     while(seleccionado == false){
       // Se genera ahora un numero aleatorio entre 0 y 1, segun
       // distribucion uniforme

       prob=General.generar_prob_uniforme();

       // Este valor de probabilidad se usa para seleccionar al
       // individuo
  
       indi=seleccionar_individuo_prob(prob);

       if (indi.acceder_seleccion() == false){

         // Se marca como seleccionado

         indi.seleccionar();

         // Se marca para final

         seleccionado=true;
       }
     }

     // Se retorna el individuo seleccionado

     return(indi);
  }

  // Metodo para asignar a cada individuo de la poblacion un valor
  // de probabilidad de acuerdo a p(1-p)^k, siendo k el lugar que
  // ocupa el individuo en la poblacion

  void asignar_probabilidad_ranking() {
    int i;
    Individuo indi;
    double prob=0;
    double prob_final;
    double sum_probs=0;
    double sum_probs_check=0;
    int numero_individuos=poblacion.acceder_numero_individuos();

      // Recorrido para todos los individuos para asignarles
      // un valor de probabilidad, segun la formula arriba
      // indicada

      for(i=0; i < numero_individuos; i++) {
         indi=poblacion.acceder_individuo(i); 

         // Se calcula el valor de probabilidad

         prob=(probabilidad_seleccion)*Math.pow(1-probabilidad_seleccion,i);

         // Se asigna este valor

         indi.asignar_probabilidad_seleccion(prob);

         // Se acumulan todos los valores de probabilidad generados

         sum_probs=sum_probs+prob;
      }


      // Para que realmente sea una probabilidad hemos de hacer otro
      // recorrido que asigne a cada individuo como valor de probabilidad
      // el que le corresponda entre la suma de probabilidades

      for(i=0; i < poblacion.acceder_numero_individuos(); i++) {
         indi=poblacion.acceder_individuo(i);

         prob=indi.acceder_probabilidad_seleccion();

         prob_final=prob/sum_probs;

         // Se reasigna la probabilidad

         indi.asignar_probabilidad_seleccion(prob/sum_probs);
         sum_probs_check+=prob/sum_probs;
      }
  }

  // Metodo que selecciona el individuo en funcion de sus
  // probabilidades. Recibe un valor de probabilidad como
  // argumento

  Individuo seleccionar_individuo_prob(double prob) {
    int i;
    double prob_indi,prob_acum=0;
    Individuo indi=null;

      // Recorrido sobre la lista de individuos

      for(i=0; i < poblacion.acceder_numero_individuos(); i++) {
         indi=poblacion.acceder_individuo(i);

         // Se accede a su probabilidad

         prob_indi=indi.acceder_probabilidad_seleccion(); 

         // Se incrementa sobre prob_acum

         prob_acum+=prob_indi;

         if (prob <= prob_acum)
             return(indi); 
      }

      // De llegar aqui indica algun error

      System.out.println("Error en seleccionar individuo seleccion");
      System.out.println("Se devuelve el ultimo de los individuos");
      return(indi); 
  }
}


