//
package genetic;

import java.util.*;
import java.io.*;

public class Poblacion {
  // Datos miembro

   // Vector con los individuos

   private Vector individuos;

   // Numero de individuos

   private int numero_individuos;

   // Valores asociados a los individuos con mayor y menor valor
   // de funcion objetivo

   private int mayor;
   private double mayor_valor_objetivo;
   private Individuo individuo_mayor_objetivo;

   private int menor;
   private double menor_valor_objetivo;
   private Individuo individuo_menor_objetivo;

   private double valor_medio_objetivo;

   // Flag para recoger la actualizacion o no de estas medidas

   private boolean mejor_actualizado;
   private boolean peor_actualizado;

   // Dato miembro privado con el objeto fitness
   // usado para calcular valor objetivo

   private fitnessMgo fitness;

  // Metodos miembro

  Poblacion(fitnessMgo fitness, int num_individuos) {
   int numero_genes;

    // Se asigna el fitness

    this.fitness=fitness;
    numero_genes=fitness.dimension();

    // Se asigna el numero de individuos

    numero_individuos=num_individuos;

    // Se crea el vector de individuos

    individuos=new Vector(num_individuos);

    // Metodo de generacion de la poblacion
    // inicial

    crear_poblacion_inicial(numero_genes);

    // Se inicializa a false el flag de actualizacion de mejor
    // y peor

    mejor_actualizado=false;
    peor_actualizado=false;

    // Se inicializa a 0 el valor de menor_valor_objetivo
    // y mayor_valor_objetivo

    menor_valor_objetivo=0;
    mayor_valor_objetivo=0;
  }

  // Para acceder al numero de individuos

  public int acceder_numero_individuos() {

     // Se refresca el numero de individuos

     numero_individuos=individuos.size();

     // Se devuelve el numero de individuos

     return(numero_individuos);
  }

  // Para acceder al individuo que ocupa una determinada
  // posicion en el vector

  public Individuo acceder_individuo(int indice) {
     return((Individuo)(individuos.elementAt(indice)));
  }

  // Para acceder a las medidas de mejor y peor fitness

  public double acceder_mayor_valor_objetivo() {
      return(mayor_valor_objetivo);
  }

  public double acceder_menor_valor_objetivo() {
      return(menor_valor_objetivo);
  }

  // Para acceder al mejor y peor individuo

  public Individuo acceder_individuo_menor_objetivo() {
     return(individuo_menor_objetivo);
  }

  public Individuo acceder_individuo_mayor_objetivo() {
     return(individuo_mayor_objetivo);
  }

  // Metodo para visualizar los individuos de la poblacion

  public void visualizar(){
    int i;
    Individuo indi;

    for(i=0; i < numero_individuos; i++){
      indi=acceder_individuo(i);
      indi.visualizar();
    }
  }

  // Metodo para deseleccionar a todos los individuos

  public void deseleccionar(){
   int i;
   Individuo indi;
   
     for(i=0; i < numero_individuos; i++){
       indi=acceder_individuo(i);
       indi.deseleccionar();
     }
  }

  private void crear_poblacion_inicial(int numero_genes) {
    int i;
    boolean valido;
    Individuo indi;

     // Se crea cada uno de los individuos

     for(i=0; i < numero_individuos; i++) {

       indi=new Individuo(numero_genes,fitness);

       // Sobre este individuo se realiza un numero aleatorio 
       // de mutaciones, buscando que todos los individuos sean
       // diferentes entre si

       indi.mutar(General.generar_ent_uniforme(1,2*numero_genes)); 

       // Se comprueba la validez del cromosoma

       valido=indi.comprobar();

       if (valido == false) {
          System.out.println("Individuo mal generado.......\n");
          System.exit(-10);
       }


       // Se inserta en el vector individuos

       individuos.add(i,indi);
     }
  }

  // Metodo para insertar un nuevo individuo en la poblacion

  public void insertar_individuo(Individuo indi) {
      individuos.add((Object)indi);

      // Hay que incrementar el numero de individuos

      numero_individuos++;

      // Al haber modificado la lista de individuos ya no
      // hay seguridad sobre la ordenacion

      mejor_actualizado=false;
      peor_actualizado=false;

      // Es necesario obtener su fitness

      indi.calcular_valor_objetivo();
  }  

  // Metodo para eliminar un individuo de la poblacion

  public void eliminar_individuo(Individuo indi) {
     boolean resultado;

     resultado=individuos.remove((Object)indi);

     //  Se decrementa el numero de individuos, si se elimino

     if (resultado == true) {
       numero_individuos--;

       // Se pone a false el flag de actualizacion

       mejor_actualizado=false;
       peor_actualizado=false;
     }
  }

  // Metodo de prueba para asignar fitness aleatorio a cada uno
  // de los individuos

  public void asignar_fitness() {
    int i;
    Individuo indi;

      // Bucle de recorrido

      for(i=0; i < numero_individuos; i++) {
        indi=(Individuo)individuos.get(i);
        indi.calcular_valor_objetivo();
      }
  }

  // Metodo para actualizar los datos miembro que recogen el mejor y
  // peor individuo de la poblacion

  public void actualizar_mejor_peor() {
     if (mejor_actualizado == false) {
        obtener_individuo_mayor_objetivo();
     }

     if (peor_actualizado == false){
        obtener_individuo_menor_objetivo();
     }
  }

  // Metodo para obtener el individuo de la poblacion con mayor valor de
  // objetivo. El individuo no se extrae del vector de individuos, simplemente
  // se actualiza el dato miembro correspondiente

  private void obtener_individuo_mayor_objetivo() {
    Individuo indi;
    double valor,media=0;
    int i;
    int ind_mayor=0;

      // Recorrido sobre el conjunto de individuos
 
      for(i=0; i < numero_individuos; i++) {
        indi=(Individuo)individuos.get(i);
        valor=indi.acceder_objetivo();
        if (i == 0){
           mayor_valor_objetivo=valor;
           individuo_mayor_objetivo=indi;
           ind_mayor=i;
        }
        else {
           //Control del menor, no se asigna directamente 
      
           if (valor > mayor_valor_objetivo) {
              mayor_valor_objetivo=valor;
              individuo_mayor_objetivo=indi;
              ind_mayor=i;
           }
        }

        // Se incrementa valor para la media

        media+=valor;
      }

      // Se almacena la media

      valor_medio_objetivo=media/numero_individuos;

      // Se pone a true el flag de actualizacion

      mejor_actualizado=true;
  }

  // Metodo para obtener el peor individuo de la poblacion. El
  // individuo no se extrae del vector de individuos, simplemente
  // se actualiza el dato miembro correspondiente

  private void obtener_individuo_menor_objetivo() {
    Individuo indi;
    double valor;
    int i;
    int ind_menor=0;

      // Recorrido sobre el conjunto de individuos
 
      for(i=0; i < numero_individuos; i++) {
        indi=(Individuo)individuos.get(i);
        valor=indi.acceder_objetivo();
        if (i == 0){
           menor_valor_objetivo=valor;
           individuo_menor_objetivo=indi;
           ind_menor=i;
        }
        else {
           //Control del menor, no se asigna directamente 
      
           if (valor < menor_valor_objetivo) {
              menor_valor_objetivo=valor;
              individuo_menor_objetivo=indi;
              ind_menor=i;
           }
        }
      }

      // Se pone a true el flag de actualizacion

      peor_actualizado=true;
  }

  // Metodo para devolver al individuo con mayor valor de funcion
  // objetivo. Este metodo extrae al individuo del vector de
  // individuos, y se usa para la seleccion mediante alguno de los
  // procedimientos implementados

  public Individuo extraer_individuo_mayor_objetivo() {

      // Se actualiza, si es necesario el valor del mejor individuo

      if (mejor_actualizado == false){
         obtener_individuo_mayor_objetivo();
      }

      // Se extrae el individuo de la poblacion

      try {
        individuos.remove(individuo_mayor_objetivo);
      } 
      catch(ArrayIndexOutOfBoundsException e) {
         System.out.println("Error en extraccion de individuo");
         System.out.println("Clase: Poblacion; Metodo: extraer_individuo_mayor_objetivo");
         System.out.println(e);
         System.exit(-10);
      }
        
      // Se decrementa el numero de individuos

      numero_individuos--;

      // Ya no est actualizado el mejor

      mejor_actualizado=false;

      // Se retorna el individuo

      return(individuo_mayor_objetivo);
  }

  // Metodo para devolver al individuo con menor valor de funcion
  // objetivo. Este metodo extrae al individuo de la poblacion, del
  // vector de individuos, de forma que se pueda usar en los 
  // procedimientos de seleccion de individuos

  public Individuo extraer_individuo_menor_objetivo() {

      // Se actualiza, si es necesario, el valor del peor individuo

      if (peor_actualizado == false){
         obtener_individuo_menor_objetivo();
      }

      // Se extrae el individuo de la poblacion

      try {
        individuos.remove(individuo_menor_objetivo);
      } 
      catch(ArrayIndexOutOfBoundsException e) {
         System.out.println("Error en extraccion de individuo");
         System.out.println("Clase: Poblacion; Metodo: extraer_individuo_menor_objetivo");
         System.out.println(e);
         System.exit(-10);
      }

      // Se decrementa el numero de individuos

      numero_individuos--;

      // Ahora ya no esta actualizado el peor

      peor_actualizado=false;

      // Se retorna el individuo

      return(individuo_menor_objetivo);
  }

  // Metodo para devolver los indices de los individuos asociados
  // con el maximo valor de funcion objetivo. La idea de devolver
  // todos los indices radica en permitir que todos ellos tengan
  // la misma probabilidad de ser elegidos

  public void obtener_indices_maximo_valor_objetivo(Vector indices) {
    Individuo indi;
    int i;

      // En primer lugar se llama al metodo de actualizado de valores
      // de mejor y peor

      actualizar_mejor_peor();

      // Una vez hecho, vemos todos los individuos que poseen el
      // valor maximo de objetivo

      for(i=0; i < individuos.size(); i++) {
          indi=(Individuo)individuos.get(i);
        
          // Se mira su valor de objetivo

          if (indi.acceder_objetivo() == mayor_valor_objetivo) {
              indices.add(new Integer(i));
          }
      } 
  }

  // Metodo para devolver los indices asociados a los individuos
  // con el minimo valor de funcion objetivo. La idea de devolver
  // los indices asociados con todos los individuos de igual valor
  // de funcion objetivo consiste en permitir que todos ellos puedan
  // ser elegidos

  public void obtener_indices_minimo_valor_objetivo(Vector indices) {
    Individuo indi;
    int i;

      // En primer lugar se llama al metodo de actualizacion de
      // valores de mejor y peor

      actualizar_mejor_peor();

      // Una vez hecho, se ve los individuos que poseen el valor
      // minimo de funcion objetivo

      for(i=0; i < individuos.size(); i++) {
          indi=(Individuo)individuos.get(i);

          // Se mira el valor de objetivo

          if (indi.acceder_objetivo() == menor_valor_objetivo) {
              indices.add(new Integer(i));
          }
      }
  }

  // Metodo que permite la ordenacion de la poblacion, segun un
  // determinado sentido: de mejor a peor o viceversa

  void ordenar_individuos(boolean objetivo) {
     Vector nuevos_individuos=new Vector();
     Individuo extraido;
     boolean max_a_min;
     int num_individuos_inicial=numero_individuos;
     int i;

     // Si hace falta, se actualizan los mejores y peores

     actualizar_mejor_peor();

     // Se han de tratar todos los individuos

     for(i=0; i < num_individuos_inicial; i++) {

        // El individuo extraido en cada fase dependera del sentido
        // de la busqueda

        if(objetivo) {
           // Estamos interesados en el individuo de mayor valor objetivo

           extraido=extraer_individuo_mayor_objetivo();
        } 
        else {
           // Estamos interesados en individuo de menor valor objetivo
           extraido=extraer_individuo_menor_objetivo();
        }
  
        // El individuo extraido se agrega al nuevo vector

        nuevos_individuos.add(extraido);
     }

     // Al finalizar se hace que este nuevo array sea el de
     // individuos

     individuos=nuevos_individuos;

     // Se restablece el numero de individuos de la poblacion

     numero_individuos=num_individuos_inicial;
  } 

  // Metodo para devolver un array de indices con las posiciones de
  // todos los individuos iguales a otro pasado como parametro

  public Vector buscar_individuo(Individuo indi) {
    Vector indices;

      // Se llama al metodo buscar_en_vector

      indices=General.buscar_en_vector(individuos,(Object)indi); 
      return(indices);
  }

  // Metodo para hacer el cruce mediante el metodo del punto

  public void cruzar_punto_individuos(Individuo padre, Individuo madre) {
     int punto_cruce;
     Individuo hijo1;
     Individuo hijo2;
     int numero_genes;
     int i;

       // En primer lugar se determina, de forma aleatoria, el punto
       // de cruce. Debe estar comprendido entre 1 y el numero de
       // genes-2. El valor da directamente la posicion del vector en
       // que se hara el corte

       numero_genes=padre.acceder_numero_genes();
       punto_cruce=General.generar_ent_uniforme(1,numero_genes);

       // Este punto de corte proporciona dos fragmentos:padre1, padre2,
       // madre1 y madre2, que se recombinan as: padre1-madre2, madre1-padre2

       hijo1=new Individuo(numero_genes,fitness);
       hijo2=new Individuo(numero_genes,fitness);

       // Preparacion de hijo 1 hijo 2

       for(i=0; i < punto_cruce; i++) {
          hijo1.asignar_gen(i,padre.acceder_gen(i));
          hijo2.asignar_gen(i,madre.acceder_gen(i));
       } 

       for(i=punto_cruce; i < numero_genes; i++) {
          hijo1.asignar_gen(i,madre.acceder_gen(i));
          hijo2.asignar_gen(i,padre.acceder_gen(i));
       }

       // Se comprueba la validez, por si hace falta reparar

       if (hijo1.comprobar() == false) {
           hijo1.reparar();
       }
       if (hijo2.comprobar() == false) {
           hijo2.reparar();
       }

       // Ambos se incluyen en la poblacion. Se marcan con edad -1

       insertar_individuo(hijo1);
       insertar_individuo(hijo2);
  }

  // Metodo para hacer el cruce por el metodo del_pozo

  public void cruzar_del_pozo(Individuo padre,Individuo madre) {
    Vector puntuaciones=new Vector();
    Vector indices=null;
    int seleccionado1;
    int seleccionado2;
    int i,j;
    int numero_genes;
    int pos_madre;
    int pos_padre;
    int pos;
    double puntuacion;
    double puntuacion_minima=0;
    int indice_minimo=0;
    Individuo hijo1;
    int primera_vez;

      numero_genes=padre.acceder_numero_genes();

      // Se crean los hijos

      hijo1=new Individuo(numero_genes,fitness);

      // Se rellena el array de puntuaciones

      for(i=0; i < numero_genes; i++) {
         // Vemos la posicion que ocupa este gen en cada uno de los
         // individuos

         pos_padre=padre.acceder_posicion_de_gen(i);
         pos_madre=madre.acceder_posicion_de_gen(i);

         puntuacion=(pos_padre/padre.acceder_objetivo())+
                    (pos_madre/madre.acceder_objetivo());

         puntuaciones.add(new Double(puntuacion));
      }

      // Se van seleccionando los elementos con puntuacion menor. Si
      // hay empate nos quedamos con el primero

      for(j=0; j < numero_genes; j++) {
         // Han de tratarse todos los genes

         primera_vez=0;
         for(i=0; i < numero_genes; i++) {
          puntuacion=((Double)puntuaciones.get(i)).doubleValue();
          if(puntuacion != -1) {
           if(primera_vez == 0) {
              puntuacion_minima=puntuacion;
              indice_minimo=i;
              primera_vez=1;
           }
           else {
              puntuacion=((Double)puntuaciones.get(i)).doubleValue();

              if (puntuacion < puntuacion_minima) {
                 puntuacion_minima=puntuacion;
                 indice_minimo=i;
              }
           }
          }
         }

         // Encontrado el minimo de esta vuelta, vemos si hay otras
         // puntuaciones repetidas

         indices=General.buscar_en_vector(puntuaciones,new Double(puntuacion_minima));

         // De este vector de indices seleccionamos al azar una de las
         // posiciones, si es necesario

         seleccionado1=indice_minimo;

         if (indices.size() > 1) {
            seleccionado1=General.generar_ent_uniforme(0,indices.size()-1);
         
            // Accedemos al gen que vamos a insertar 

            seleccionado1=((Integer)indices.get(seleccionado1)).intValue();
         }

         // Las posiciones seleccionadas se corresponden con los genes
         // que van en la posicion j

         hijo1.asignar_gen(j,new Integer(seleccionado1));

         // En todas las posiciones de puntuaciones ya tratadas se pone -1

         puntuaciones.set(seleccionado1,new Double(-1));
      }

      // Se comprueba la validez, por si hace falta reparar

      if (hijo1.comprobar() == false) {
          System.out.println("Error en generacion del pozo");
          System.exit(0);
      }

      // se incluye en la poblacion. Se marca con edad -1

      insertar_individuo(hijo1);
  }

  // Metodo para actualizar las edades de los individuos de la poblacion

  public void actualizar_edad_individuos() {
    int i;
    int edad;
    Individuo indi;

      // Recorrido sobre la lista de individuos
   
      for(i=0; i < numero_individuos; i++) {
        indi=(Individuo)individuos.get(i);
        indi.incrementar_edad();
      } 
  }

  // Metodo para listar los objetivos de todos los individuos

  private void listar_objetivos() {
    int i;
    Individuo indi;
  
      System.out.println("-------------------------------------------------"); 
      for(i=0; i < numero_individuos; i++) {
         indi=acceder_individuo(i);
         System.out.println("Ind (" + i + ") = "+ indi.acceder_objetivo()); 
      System.out.println("-------------------------------------------------"); 
      }
  }
}
