//
package lbnk2;

//
import kbm2l.*;

//
import java.util.*;
import java.lang.*;

/**
 * Title: K2.java
 * Description:  K2 algorithm------learning Bayes Networks from data
 * *	Input:   An input datafile which contains
 *                <1> a set of n nodes
 *		  <2> an ordering on the nodes
 *		  <3> an upper bound u on the number of parents a node may have
 *		  <4> a database D contains m cases
 *	Output:	 Learn the parent nodes for each nodes and return an XMLBIF
 *               file representing the learned Bayesian Network
 * Copyright:    Copyright (c) 2001
 * Company:      KSU / KDD
 * @author Haipeng Guo
 * @version 0.2.0
 */

public  class  K2  extends  frame30 {

  private LearningInputDataManager data; // input data for learning
  private LearningBBN learnedBBN = new LearningBBN();
  private int u = 5;  // upper bound of max number of parents for each node

   /**
   * K2 Constructor receives the input data
   */

  public K2(LearningInputDataManager inputdata){
	   data = inputdata;
  }

  /**
   * setOrdering sets the ordering in K2.data
   */
  public void setOrdering(Vector alpha) {
    this.data.setOrdering(alpha);
  }

  /**
   * learnBBNFromData learns the BBN from data .
   * @return LearningBBN - the learned Bayesian Network
   */

  public LearningBBN learnBBNFromData() {

    // declaretions about K2
    // m --- there are m cases in the database
    // n --- there are n variables or nodes
    // u --- an upper bound on the number of parents a node may have
    int m =0;
    int n = 0;
    boolean OkToProceed = true;
    double Pnew = 0;
    double Pold = 0;

   // *******initialization stage*********

   // Z is the set of all nodes,  Z is loaded from input database

    f30.kbm2lSystem.append("\nK2 Algorithm----Learning Bayes Networks from Data\n");
//    System.out.println("\nK2 Algorithm----Learning Bayes Networks from Data");
    //System.out.println("\nK2 Algorithm----Learning Bayes Networks from Data");
    f30.kbm2lSystem.append("\n******Start initialization process, get data from database......\n");
//    System.out.println("\n******Start initialization process, get data from database......");
    LearningBBN Z_allNodes = new LearningBBN(data);
    // there are n nodes and m cases in the database
    n = data.getNumberOfNodes();
    m = data.getNumberOfCases();
    f30.kbm2lSystem.append("The database has "+n +" variables(nodes)\n");
//    System.out.println("The database has "+n +" variables(nodes)");
    f30.kbm2lSystem.append("There are "+m +" cases in the database\n");
//    System.out.println("There are "+m +" cases in the database");
    Z_allNodes.print_nodes();
    Z_allNodes.printparentlist();
    f30.kbm2lSystem.append("\n******Initialization finished, start to search for the strcture......\n");
//    System.out.println("\n******Initialization finished, start to search for the strcture......");

    //	*******Searching stage (K2) ********

    LearningNode current_node = new LearningNode();
    int numOfParents = 0;
    String curnodename;
    String z;
    Vector searchList = new Vector();
    Vector tempoldparents = new Vector();
    Vector tempnewparents = new Vector();
    // the main searching loop, it searchs parents nodes for each node
    for(int i=0;i<n;i++){
      current_node = Z_allNodes.getNodeAt(i);
      numOfParents = current_node.getParentlist().size();
      curnodename = current_node.getNodeName();
      f30.kbm2lSystem.append("\ncurrent node----------------------------------"+curnodename+"\n");
//      System.out.println("\ncurrent node----------------------------------"+curnodename);
      f30.kbm2lSystem.append(current_node.getAllValues()+"\n");
//      System.out.println(current_node.getAllValues());

      current_node.empty_parentlist();
      tempoldparents.removeAllElements();
      OkToProceed = true;
      for(int s=0;s<current_node.getParentlist().size();s++){
        tempoldparents.addElement(current_node.getParentlist().elementAt(s).toString());
        }
      // calculate Pold using old parents
      Pold = g(m, current_node, tempoldparents);
      f30.kbm2lSystem.append("Pold="+Pold+"\n");
//      System.out.println("Pold="+Pold);
      while((OkToProceed ) && (current_node.getParentlist().size() < u)){
	Vector predlist = new Vector();
	predlist = Z_allNodes.pred(current_node.getNodeName());
	// get the searchlist
	searchList = sub(predlist,tempoldparents);
          if(searchList.isEmpty()){break;}
	// z is the best node in the searchList that maximizes g(i, (Pi\/z))
	z = maxNode( m, current_node, tempoldparents, searchList);
	f30.kbm2lSystem.append("MaxNode in searchlist is "+z+"\n");
//	System.out.println("MaxNode in searchlist is "+z);
	tempnewparents.removeAllElements();
	for(int r=0;r<tempoldparents.size();r++){
          tempnewparents.addElement(tempoldparents.elementAt(r).toString());
	}
	tempnewparents.addElement(z);

	Pnew = g(m, current_node, tempnewparents);
	f30.kbm2lSystem.append("Pnew = "+Pnew+"\n");
//	System.out.println("Pnew = "+Pnew);
	f30.kbm2lSystem.append("Pold = "+Pold+"\n");
//	System.out.println("Pold = "+Pold);
	if(Pnew > Pold){
          Pold = Pnew;
	  tempoldparents.addElement(z);
	  f30.kbm2lSystem.append(
	    "Pnew > Pold, add node "+ z +" to Parentlist of "+current_node.getNodeName()+"\n");
//	  System.out.println("Pnew > Pold, add node "+ z +" to Parentlist of "+current_node.getNodeName());
	}
	else {
          OkToProceed = false;
	}
      }  // end of while
	//System.out.println("end of ---------------------------------------"+curnodename);
      for(int t=0;t<tempoldparents.size();t++){
	current_node.add_parentlist(tempoldparents.elementAt(t).toString());
      }
  }  // end of for


  f30.kbm2lSystem.append("******searching finished, print the result structure......\n");
//  System.out.println("******searching finished, print the result structure......\n");
  Z_allNodes.printparentlist();

  f30.kbm2lSystem.append("\n******computing the conditional probability table......\n");
//  System.out.println("\n******computing the conditional probability table......\n");
  Z_allNodes = learnCPTFromData(Z_allNodes);
  for(int t=0;t<Z_allNodes.getNumberOfNodes();t++){
    f30.kbm2lSystem.append("******node:"+Z_allNodes.getNodeAt(t).getNodeName()+"******\n");
//    System.out.println("******node:"+Z_allNodes.getNodeAt(t).getNodeName()+"******");

    Z_allNodes.getNodeAt(t).printparents();
    f30.kbm2lSystem.append("Probability table...\n");
//    System.out.println("Probability table...");
    f30.kbm2lSystem.append(Z_allNodes.getNodeAt(t).getCPT().toString()+"\n");
//    System.out.println(Z_allNodes.getNodeAt(t).getCPT().toString());
  }
  return Z_allNodes;

  } // end of search

  private double g(int m, LearningNode ith_node,Vector parentNodelist){
    Vector p_list = new Vector();
    //System.out.print("p_list:");
    for(int i=0;i<parentNodelist.size();i++){
      p_list.addElement(parentNodelist.elementAt(i));
      //System.out.print(p_list.elementAt(i).toString()+" ");
    }
		//System.out.println("");

    double result = 0;
    double sum1=0;
    double sum2=0;
    double sum3=0;
    int qi;
    int ri = ith_node.getArity();
    ParentNodes p = new ParentNodes(data, p_list);
    if(p_list.size()==0){
      qi=0;
    }
    else{
      qi = p.getQ();
    }

    if(qi>0){
      for(int j=1;j<=qi;j++){
        sum1 = ln_factorial(ri-1);
        sum2 = ln_factorial(N_ij(m, ith_node, p, j)+ri-1);
        sum3 = 0;
        for(int k=1;k<=ri;k++){
          sum3 = sum3 + ln_factorial(N_ijk(m, ith_node, p, j, k));
        }
        result = result + sum1 - sum2 + sum3;
      }
    }
    if(qi==0){
      sum1 = ln_factorial(ri-1);
      sum2 = ln_factorial(N_ij(m, ith_node, p, 0)+ri-1);
      sum3 = 0;
      for(int k=1;k<=ri;k++){
        sum3 = sum3 + ln_factorial(N_ijk(m, ith_node, p, 0, k));
      }
			//System.out.println("qi==0");
    result =  sum1 - sum2 + sum3;
    };
  //System.out.println("g:"+ result);
    return result;
  }

  private int N_ijk(int m, LearningNode ith_node,ParentNodes parentnodelist,int j, int k){
    int result = 0;
    boolean b1 = true;
    boolean b2 = false;
    for(int h=1;h<m+1;h++){
      if (ith_node == null) {
        System.out.println("ith_node == null");
        }
      b1 = ith_node.getValueAtRow(h).equals(ith_node.get_Kth_v(k));
      if(parentnodelist.isEmpty())
	b2= true;
      else{
        b2 = parentnodelist.get_jth_from_v(j).equals(parentnodelist.getParentnodesAtRow(h));}

      if(b1&&b2) {result++;}
    }
    return result;
  }

  private int N_ij(int m, LearningNode ith_node,ParentNodes parentnodelist,int j){
    int result = 0;
    int ri = ith_node.getArity();
    for(int k=1;k<=ri;k++){
      result = result + N_ijk(m, ith_node, parentnodelist, j, k);
    }
		//System.out.println("Ni"+j+":"+result);
    return result;
  }

  private double ln_factorial(int n){
    double result = 0;
    for(int i=1;i<=n;i++){
      result = result+ Math.log(i);
    }
    return result;
  }


  // return a nodelist v = v1 - v2
    private Vector sub(Vector v1, Vector v2){
      int v2_nodes_num = v2.size();
      for(int j=0;j<v2_nodes_num;j++){
        v1.removeElement(v2.elementAt(j));
      }
    return v1;
  }

    private String maxNode(int m, LearningNode ith_node,Vector parentNodelist, Vector candidateNodelist){
      String result="";
      double old_g = 0;
      double new_g = 0;
      int j=0;

      Vector p_list = new Vector();
      Vector c_list = new Vector();

      if(candidateNodelist.size()==1){
        result = candidateNodelist.elementAt(0).toString();
        return result;
      }
      for(int i=0;i<parentNodelist.size();i++){
        p_list.addElement(parentNodelist.elementAt(i));
      }
      for(int i=0;i<candidateNodelist.size();i++){
        c_list.addElement(candidateNodelist.elementAt(i));
      }
      int candidate_num = c_list.size();

	  // when i=0, avoid old_g = 0, for it may be <0
      p_list.addElement(c_list.elementAt(0));
      new_g = g(m, ith_node, p_list);
      old_g = new_g;
      p_list.removeElement(p_list.lastElement());

	  // when i>0
      for(int i=1;i<candidate_num;i++){
        p_list.addElement(c_list.elementAt(i));
		  //System.out.println("testing :"+c_list.elementAt(i).toString());
        new_g = g(m, ith_node, p_list);
		  //System.out.println("old_g="+old_g);
		  //System.out.println("new_g="+new_g);
        if(new_g > old_g){ //System.out.println("new_g>old_g...");
          result = c_list.elementAt(i).toString();
          old_g = new_g;
          j++;
        }
        p_list.removeElement(p_list.lastElement());
      }

    if(j==0){
      result = c_list.elementAt(0).toString();
	//System.out.println("first node is max..."+result);
    }
    return result;
  }

// compute CPT table

  private LearningBBN learnCPTFromData( LearningBBN all_nodes) {
    double theta = 0;
    int m;
    int n;
    int qi;
    int ri;
    int Nijk;
    int Nij;
    m = data.getNumberOfCases();
    n= all_nodes.getNumberOfNodes();
    LearningNode current_node;
    Vector p_list = new Vector();
    for(int i=0;i<n;i++){
      p_list.removeAllElements();
      current_node = all_nodes.getNodeAt(i);
      f30.kbm2lSystem.append("current_node----------"+current_node.getNodeName()+"\n");
//      System.out.println("current_node----------"+current_node.getNodeName());
     f30.kbm2lSystem.append("It's parents: ");
//      System.out.print("It's parents: ");
      current_node.printparents();
      for(int s=0;s<current_node.getParentlist().size();s++){
        p_list.addElement(current_node.getParentlist().elementAt(s).toString());
      }
      ParentNodes p = new ParentNodes(data, p_list);
      qi = p.getQ();
      if(qi==0){qi++;};
      ri = current_node.getArity();
      for(int k=1;k<=ri;k++){
              //		System.out.println("N"+(i+1)+j+"="+Nij);
        for(int j=1;j<=qi;j++){
            Nij = N_ij(m, current_node, p, j);
            Nijk = N_ijk(m, current_node, p, j, k);
					//	 System.out.println("N"+(i+1)+j+k+"="+Nijk);
            theta = ((double)Nijk+1)/(Nij+ri);
            all_nodes.getNodeAt(i).addElementToCPT(Double.toString(theta));
						 //System.out.println("THETA"+(i+1)+j+k+"="+all_nodes.getNodeAt(i).getCPTElementAt(j-1));
        }
      }
    }
    return all_nodes;
  }
}
 // end of K2
