package lbnk2;

//
import java.util.*;
import java.lang.*;
import java.io.*;

//
import tables.*;

/**
 * Title: LearningBBN.java
 * Description:  Core Bayesian Network Learning class
 * Copyright:    Copyright (c) 2001
 * Company:      KSU / KDD
 * @author Haipeng Guo
 * @author Laura Kruze
 * @author Ben Perry
 * @author Julie Stilson
 *
 * @version 0.2.0
 */

public class LearningBBN
{
  private int n = 0;  // number of nodes
  private int m = 0;  // number of cases
  private Vector nodeList = new Vector();  // vector of all nodes

   /**
   * LearningBBN: initialize nodelist, n from LearningInputDataManager.
   */

  public LearningBBN(){
	}

  public LearningBBN(LearningInputDataManager data){
    String varname = "";
    for(int i=0;i<data.getNumberOfNodes();i++){
      varname = data.getOrdering().elementAt(i).toString();
      System.out.print("i="+i);
      System.out.println(" var_name:"+varname);
      LearningNode node1 = new LearningNode(varname,data);
      nodeList.addElement(node1);
    }
    n = data.getNumberOfNodes();
    m = data.getNumberOfCases();
  }

  public int getNumberOfNodes(){
    return this.n;
  }

  public int getNumberOfCases(){
    return this.m;
  }

  public LearningNode getNodeAt(int i){
    LearningNode node1;
    Object obj = new Object();
    obj = this.nodeList.elementAt(i);
    node1 = (LearningNode) obj;
    return node1;
  }

  public LearningNode getNodeByName(String name) {
	for (int i = 0; i < nodeList.size(); i++) {
		LearningNode n = (LearningNode) nodeList.elementAt(i);
		if (n.getNodeName().equals(name)) {
			return n;
		}
	}
	return null;
  }

  public void print_r(){
    LearningNode node1;
    Object obj = new Object();
    for(int i=0;i<n;i++){
      obj = this.nodeList.elementAt(i);
      node1 = (LearningNode) obj;
      System.out.print(node1.getNodeName()+" ");
    }
    System.out.println("");
    for(int i=0;i<n;i++){
      obj = this.nodeList.elementAt(i);
      node1 = (LearningNode) obj;
      System.out.print(node1.getArity() +" ");
    }
 }

  public void writeToXML(String outputXMLFileName){
    System.out.println("write to an XML file!");
    String listall = "";
    listall = listHead(dato30.nameFile(outputXMLFileName)) + listNode() +
      "\n<!-- Probabilitydistributions-->\n" + 
      listProbability() + 
      "\n\n\n"+"</NETWORK>"+"\n"+"</BIF>";
    try{
      BufferedWriter bw=new BufferedWriter(new FileWriter(outputXMLFileName));
      bw.write(listall,0,listall.length());
      bw.close();
      }
      catch(IOException e){
        System.out.print("An IOException occured");
      }
 }

  private String listNode(){
    String name;
    int i=0;
    String listA="";
    for(i=0; i< this.getNumberOfNodes(); i++){
      name = this.getNodeAt(i).getNodeName();
      listA =listA+"<VARIABLE TYPE=\"nature\">"+"\n\t"+"<NAME>"+name+"</NAME>\n";
      for(int j=0;j<this.getNodeAt(i).getArity();j++){
        listA = listA +"\t<OUTCOME>" + this.getNodeAt(i).get_Kth_v(j+1)+"</OUTCOME>\n";
      }
      listA = listA +listPosition(i)+"</VARIABLE>\n\n";
    }

    return listA;
  }

  private String listPosition(int i){
    String listC="";
    int x=i;
    int y=i;
    listC="\t<PROPERTY>position = ("+ x + ", "+ y + ")</PROPERTY>\n";
    return listC;
}

  private String listProbability(){
    String listP="";
    for( int i = 0; i <this.getNumberOfNodes(); i++){
      listP = listP + "<DEFINITION>\n"+"\t<FOR>"+this.getNodeAt(i).getNodeName()+"</FOR>\n";
      for(int j=0;j<this.getNodeAt(i).getParentlist().size();j++){
        listP = listP + "\t<GIVEN>" + this.getNodeAt(i).getParentlist().elementAt(j) +"</GIVEN>\n";
      }
      listP = listP + "\t<TABLE>";
      for(int k=0;k<this.getNodeAt(i).getCPT().size();k++){
        listP = listP + this.getNodeAt(i).getCPTElementAt(k)+ " ";
      }
      listP = listP + "</TABLE>\n";
      listP = listP + "</DEFINITION>\n\n";
    }
    return listP;
  }


	/**
	 * list the first(head) part of the xml file
	 */
  private String listHead(String outputXMLFileName){
    String list = "";
    list  = ""+"<?xml version=\"1.0\" encoding=\"US-ASCII\"?>\n\n\n"+
            "<!--"+"\n\t"+"Bayesian network in XMLBIF v0.3 (BayesNet Interchange Format)"+
            "\n\t"+"Produced by JavaBayes (http://www.cs.cmu.edu/~javabayes/"+
            "\n\t"+"Output created Tue May 02 16:32:10 CDT 2000\n"+"-->\n\n\n\n"+
            "<!-- DTD for the XMLBIF 0.3 format -->\n"+
            "<!DOCTYPE BIF ["+
            "\n\t"+"<!ELEMENT BIF ( NETWORK )*>"+
            "\n\t\t"+"<!ATTLIST BIF VERSION CDATA #REQUIRED>\n"+
            "\t"+"<!ELEMENT NETWORK ( NAME, ( PROPERTY | VARIABLE | DEFINITION )*)>\n"+
            "\t"+"<!ELEMENT NAME (#PCDATA)>\n"+
            "\t"+"<!ELEMENT VARIABLE ( NAME, ( OUTCOME |  PROPERTY )* ) >\n"+
            "\t\t"+"<!ATTLIST VARIABLE TYPE (nature|decision|utility) \"nature\">\n"+
            "\t"+"<!ELEMENT OUTCOME (#PCDATA)>\n"+
            "\t"+"<!ELEMENT DEFINITION ( FOR | GIVEN | TABLE | PROPERTY )* >\n"+
            "\t"+"<!ELEMENT FOR (#PCDATA)>\n\t"+"<!ELEMENT GIVEN (#PCDATA)>\n\t"+
            "<!ELEMENT TABLE (#PCDATA)>\n\t"+"<!ELEMENT PROPERTY (#PCDATA)>\n"+
            "]>\n\n\n"+"<BIF VERSION=\"0.3\">\n"+"<NETWORK>\n"+
            "<NAME>"+outputXMLFileName+"</NAME>\n\n"+"<!-- Variables -->\n";
	 //System.out.println(list);
    return list;
  }

  public void print_nodes(){
    LearningNode node1;
    Object obj = new Object();
    System.out.println("There are totally " + n + " nodes.");
    for(int i=0;i<n;i++){
      obj = this.nodeList.elementAt(i);
      node1 = (LearningNode) obj;
      System.out.println(node1.getNodeName());
    }
  }

  public void printparentlist(){
    System.out.println("");
    System.out.println("print all nodes's parent nodes......");
    for(int i=0;i<this.n;i++){
     System.out.println("node "+ i + ":"+this.getNodeAt(i).getNodeName());
     System.out.println("'s parent nodes are :");
     this.getNodeAt(i).printparents();
    }
  }

  /**
   * pred will return a list of nodes that are before the given node in the node order.
   *
   * @param nodename - the name of the node being requested.
   * @return previousNodes - all nodes before the node being requested in the node order.
   *
   */
  public Vector pred(String nodename){
    Vector previousNodes = new Vector();
    LearningNode node1;
    for(int i=0;i<this.n;i++){
      node1 = (LearningNode)this.nodeList.elementAt(i);
      if(node1.getNodeName().equals(nodename)){
        return previousNodes;
      }
      else {
        previousNodes.addElement(node1.getNodeName());
      }
    }
    return previousNodes;
  }

}
