package lbnk2;

//
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.io.*;

/**
 * Title: LearningInputDataManager.java
 * Description:  Core Bayesian Network class for BBN Learning
 * Copyright:    Copyright (c) 2001
 * Company:      KSU / KDD
 * @author Haipeng Guo
 * @author Ben Perry
 * @author Julie Stilson
 *
 * @version 0.2.0
 */

public class LearningInputDataManager {

  private String inputDataFileName="";
  private int nodenumber;                    // number of nodes of the learned BBN
  private int casenumber;                    // number of cases in the input data file
  private Vector aritieslist = new Vector();  // how many values that each node can take
  private Vector orderlist = new Vector();    // an ordering of nodes, parents appear before children
  private String[][] dataArray ;              // store all data points

  /**
   * LearningInputDataManager constructs and initializes the LearningInputData Manager.
   * @param filename -  the input data file name
   */

  public LearningInputDataManager(String filename) {
    inputDataFileName = filename;
  }

  public LearningInputDataManager(){
  }

  public void LoadInputFileDialog() {
    JFrame frame = new JFrame("Learning BBN from Data...");
    FileDialog fd = new FileDialog(frame,"get the input data...");
    frame.show();
    fd.show();
    String file1="";
    String path1="";
    file1 = fd.getFile();
    path1 = fd.getDirectory();
    System.out.println("file name is: "+path1+file1);
    this.inputDataFileName = path1+file1;
    frame.hide();
  }

   /**
   * loadInputDataFile reads the InputData file, loads data from the InputData file into the corresponding variables.
   * @param inputfilename -  the input data file name
   */

   public void loadInputDataFile(){
    try{
      BufferedReader inputfile =new BufferedReader(new FileReader(this.inputDataFileName));
      System.out.println("inputfile: "+inputDataFileName);
      String templine = "";
      int i=0; // line number indicator
      int k=0;
      while(inputfile.ready()){
      templine = inputfile.readLine().trim();
      i++;
      // first line of the input datafile---an integer
      // representing the number of nodes in the BBN to be learned
      if(i==1) {
        System.out.println("i="+i);
	nodenumber = new Integer(templine).intValue();
	System.out.println("nodenumber:"+nodenumber);
      }
      // second line of the input datafile---an array of Strings
      // representing an ordering of names of all nodes, seperated by "|"
      if(i==2){
        StringTokenizer t1 = new StringTokenizer(templine,"|");
        System.out.println("i="+i);
        int count = t1.countTokens();
        for(int j=0;j<count;j++) {
          this.orderlist.addElement(t1.nextToken());
        }
        for(int j=0;j<orderlist.size();j++)
          System.out.print(orderlist.elementAt(j)+"  ");
        }

        // third line of the input datafile---an array of numbers
        // representing number of possible values of all nodes, seperated by "|"
	if(i==3){
          StringTokenizer t2 = new StringTokenizer(templine,"|");
	  int count = t2.countTokens();
	  System.out.println("i="+i);
	  for(int j=0;j<count;j++)
	    aritieslist.addElement(t2.nextToken());
          for(int j=0;j<aritieslist.size();j++)
	    System.out.print(aritieslist.elementAt(j)+"  ");
        }
        // 4th line of the input datafile---an integer m
        // representing the number of cases in the data file
        // the input file has 4+m lines
        if(i==4){
          System.out.println("i="+i);
          casenumber = new Integer(templine).intValue();
          System.out.println("lines =  "+casenumber);
          System.out.println("nodes =  "+nodenumber);
          dataArray = new String[casenumber][nodenumber];
        }
        // from 5th line to the end of input datafile
        // m data points to be loaded into dataArray[][]
	if(i>4){
          StringTokenizer t2 = new StringTokenizer(templine," ");
          int count = t2.countTokens();
	  for(int j=0;j<count;j++)
	    dataArray[k][j]=t2.nextToken();
	  k++;
	}
      }
    } catch (Exception ex){System.err.println("Exception:) " + ex.getMessage());};
  }

   /**
   * setInputFileName sets the input data file name.
   *
   */
   public void setInputFileName(String filename){
    this.inputDataFileName = filename;
   }

   /**
   * getCaseNumber returns the number of cases.
   * @return int -  number of cases of the input data
   */
   public int getNumberOfCases(){
    return casenumber;
  }

  /**
   * getNodeNumber returns the number of nodes.
   * @return int -  number of nodes of the input data
   */
   public int getNumberOfNodes(){
    return nodenumber;
  }

   /**
   * getOrdering returns the ordering of all nodes.
   * @return orderlist -  an ordering of all variables in the input data
   */
   public Vector getOrdering(){
    return orderlist;
   }

   /**
   * setOrdering sets the ordering of all nodes.
   * @param alpha - a vector of strings(nodenames) indicating the ordering of the nodes
   */
   public void setOrdering(Vector alpha){

     this.orderlist = alpha;
  }

   /**
   * getOrdering returns the arities of all nodes.
   * @return aritieslist -  an ordering of all variables in the input data
   */
        public Vector getArities(){
          return aritieslist;
        }

   /**
   * getDataAt returns a string in the input datafile at row x, column y.
   * @return dataArray[x][y] -  a string of value at data point [x, y]
   */
        public String getDataAt(int x, int y){
          return this.dataArray[x][y];
        }


   /**
   * getColumn returns the column number of the given variable.
   * @param variable_name -  name of the variable
   * @return int - column number of the variable
   */

  public int getColumn(String variable_name){
    int column = 0;
    String temp = variable_name;
    column = orderlist.indexOf(temp)+1;
    return column;
  }

}
