//
package lbnk2;

//
import java.util.*;
import java.lang.*;

//
import kbm2l.*;

/**
 * Title: LearningNode.java
 * Description:  Core Bayesian Network Learning class
 * Copyright:    Copyright (c) 2001
 * Company:      KSU / KDD
 * @author Haipeng Guo
 * @version 0.2.0
 */

 public class LearningNode extends frame30 {

  private static final String ERROR = "Error(LearningNode): ";

  private String nodename;
  private int r = 0;					  // variable(node) may have r possible values
  private Vector v = new Vector();  // all possible values
  private Vector parentlist = new Vector();
  private Vector probabilitytable = new Vector();
  private Vector allvalues = new Vector();   // the column of data of that node in the input data file

  /**
   * LearningNode constructs the node and initializes the values from LearningInputDataManager
   * @param variable_name - the name of the node being initialized.
   *
   */

  public LearningNode(){
  }

  public LearningNode(String variable_name, LearningInputDataManager data){
    nodename = variable_name;
    f30.kbm2lSystem.append("enter Node...\n");
//    System.out.println("enter Node...");
    int column = data.getColumn(variable_name);
    f30.kbm2lSystem.append("column:" + column + "\n");
//    System.out.println("column:"+column);
    String element="";

    for(int i=0;i<data.getNumberOfCases();i++){
      element = data.getDataAt(i, column-1);
      allvalues.addElement(element);
      //System.out.println("i="+i);
      if(!(v.contains(element))){
        v.addElement(element);
        r++;
      }
    }
  } // end of constuctor

  /**
   * getNodeName will return the nodename.
   * @return value - the nodename.
   */
  public String getNodeName(){
    return this.nodename;
  }

  /**
   * getArity will return r.
   * @return value - r(the arity of that node).
   */
  public int getArity(){
    return this.r;
  }

   /**
   * getArity will return r.
   * @return value - r(the arity of that node).
   */
  public Vector getParentlist(){
    return this.parentlist;
  }

   /**
   * getAllValues will return v.
   * @return value - v(all possible values of that node).
   */
  public Vector getAllValues(){
    return this.v;
  }
   /**
   * getCPT will return probabilitytable.
   * @return value - probabilitytable.
   */
  public Vector getCPT(){
    return this.probabilitytable;
  }
   /**
   * getCPTElementAt will return the ith number in the probabilitytable.
   * @param   - the index of the required probability
   * @return  - the ith number in the probabilitytable.
   */
  public String getCPTElementAt(int i){
    return this.probabilitytable.elementAt(i).toString();
  }

  /**
   * getValueAtRow will return the value of that node at the specified row in input data.
   * @param index - row number of the node being requested.
   * @return value - value of the node at the specified row in input data.
   *
   */

  public String getValueAtRow(int index)	{
    String value ="";
    try {    
    if ( allvalues == null) {
      System.out.println("allvalues == null");
      }
    if ((index-1 >= allvalues.size()) || (index-1 < 0)) {
      System.out.println("index-1: " + (index-1) + " >= " + allvalues.size());
      }  
    value = allvalues.elementAt(index-1).toString();
    } catch (Exception e) {
      System.out.println( ERROR + "getValueAtRow().");
      System.out.println(nodename);
      System.out.println(allvalues);
      System.out.println("index-1: " + (index-1) + " >= " + allvalues.size());
      e.printStackTrace();
      System.exit( 0);
    }//catch

    return value;
  }

  /**
   * get_Kth_v will return the Kth value of all possible values.
   * @param index - index of the value being requested.
   * @return value - node value at the specified index of possible values.
   *
   */
  public String get_Kth_v(int index){
    String result="";
    result = v.elementAt(index-1).toString();
    return result;
  }

  /**
   * empty_parentlist will empty the node's parentlist.
   *
   */
  public void empty_parentlist(){
    parentlist.removeAllElements();
    f30.kbm2lSystem.append("Empty parent list of "+ nodename + "\n");
//    System.out.println("Empty parent list of "+ nodename);
  }

  /**
   * add_parentlist will add one node to the node's parentlist.
   * @param node1 - node1
   */
  public void add_parentlist(String node1){
    parentlist.addElement(node1);
    f30.kbm2lSystem.append("Add to parent list of "+ nodename+": " + node1 + "\n");
//    System.out.println("Add to parent list of "+ nodename+": " + node1);
  }

  /**
   * addElementToCPT will add one number to the node's probabilitylist.
   * @param node1 - prob
   */
  public void addElementToCPT(String prob){
    this.probabilitytable.addElement(prob);
  }

  /**
   * removelast_from_prarentlist will remove the last node from the node's parentlist.
   *
   */
  public void removelast_from_prarentlist(){
    parentlist.removeElement(parentlist.lastElement());
    f30.kbm2lSystem.append("Remove last element from parent list of " + nodename + "\n");
//    System.out.println("Remove last element from parent list of "+ nodename);
  }

  public void printparents(){
    if(this.parentlist.isEmpty()){
      f30.kbm2lSystem.append("--empty!\n");
//      System.out.println("--empty!");
    }
    else
      f30.kbm2lSystem.append("--"+this.parentlist.toString()+"\n");
//      System.out.println("--"+this.parentlist.toString());
  }//printparents
  
}//class LearningNode
