//MASKTABLE30//
package tables;

//
import java.util.*;
import java.lang.*;
import java.io.*;
import java.math.*;
import java.awt.*;

//
import control.*;
import kbm2l.*;
import kernel.*;

/**
* Clase maskTable30.
* Mascara fisica de las tablas.
* @author Juan A. Fdez del Pozo de Salamanca
* @version 3.0
**/
public  class  maskTable30   {

  private static final String ERROR = "Error(maskTable30): ";

/**
* filtra la medida
**/
  private  static  boolean wr = false; 
/**
* filtra componentes de la medida
* @see #selMaskr(int,Vector,Vector)
**/
  private  static  Vector maskr = null;
/**
* filtra la respuesta
**/
  private  static  boolean ws = true;
/**
* filtra componentes de la respuesta
* @see #selMask(int,Vector,Vector)
**/
  private  static  Vector masks = null;
/**
* filtra el indice
**/
  private  static  boolean wt = true;
/**
* filtra componentes o atributos del indice
* @see #selMask(int,Vector,Vector)
**/
  private  static  Vector maskt = null;
/**
* filtra el fin de linea estandar para los learning data sets.
**/
  public boolean wend = false;

  private static boolean projection = false;

/**
* Constructor por defecto.
**/
public maskTable30( lista30 l30) {

  try {
  wr = new controlPrefs30( ).subKBMeasure( );
  maskr = new Vector();
  for( int i = 0; i < l30.ParMedida(); i++)
    maskr.add( new Integer( -1));//

  ws = new controlPrefs30( ).subKBResponse( );
  masks = new Vector();
  for( int i = 0; i < l30.ParRespuesta(); i++)
    masks.add( new Integer( -1));//

  wt = new controlPrefs30( ).subKBIndex( );
  maskt = new Vector();
  for( int i = 0; i < l30.dimension(); i++)
    maskt.add( new Integer( -1));//

  wend = false;

  this.projection = new controlPrefs30( ).subKBProjection( );

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "maskTable30.0 ");
    e.printStackTrace();
    System.exit(0);
  }//catch
//  ver( "maskTable30.0");
}//maskTable30

/**
* Constructor.
**/
public maskTable30( lista30 l30, 
  Vector maskr, Vector masks, Vector maskt, 
  boolean wend) {

  try {
  this.wr = new controlPrefs30( ).subKBMeasure( );
  if (wr)
    this.maskr = ((Vector)maskr.clone());

  this.ws = new controlPrefs30( ).subKBResponse( );
  if (ws)
    this.masks = ((Vector)masks.clone());

  this.wt = new controlPrefs30( ).subKBIndex( );
  if (wt)
    this.maskt = ((Vector)maskt.clone());

  this.wend = wend;

  this.projection = new controlPrefs30( ).subKBProjection( );

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "maskTable30.1 ");
    e.printStackTrace();
    System.exit(0);
  }//catch
//  ver( "maskTable30.1");
}//maskTable30

/**
* Seleccion de entradas por valores de medida r.
* -1: pasa, x pasa si es igual INDEX[i] a mask[i]
**/
public static boolean selMaskr( Vector INDEX) {

  int i = 0;
  try {
    if (!wr) return false;
    for( i = 0; i < INDEX.size(); i++)
      if (((Integer)maskr.get( i)).intValue() != -1)
        if (((Double)maskr.get( i)).doubleValue() != ((Double)INDEX.get( i)).doubleValue())
          return false;
    }//try
  catch (Exception e) {
    System.out.println( ERROR + "selMaskr. " + "i:" + i);
    System.out.println( maskr);
    System.out.println( INDEX);
    e.printStackTrace();
    System.exit(0);
  }//catch
  
  return true;
}//selMaskr

/**
* Seleccion de entradas por valores de la respuesta o el indice. 
* -1: pasa, x pasa si es igual INDEX[i] a mask[i]
**/
public static boolean selMasks( Vector INDEX) {

  int i = 0;
  try {
    if (!ws) return false;
    for( i = 0; i < INDEX.size(); i++)
      if (((Integer)masks.get( i)).intValue() != -1)
        if (((Integer)masks.get( i)).intValue() != ((Integer)INDEX.get( i)).intValue())
        return false;
    }//try
  catch (Exception e) {
    System.out.println( ERROR + "selMask. " + "i:" + i);
    System.out.println( masks);
    System.out.println( INDEX);
    e.printStackTrace();
    System.exit(0);
  }//catch
  
  return true;
}//selMasks

/**
* Seleccion de entradas por valores del indice. 
* -1: pasa, x pasa si es igual INDEX[i] a mask[i]
**/
public static boolean selMaskt( Vector INDEX) {

  int i = 0;
  try {
    if (!wt) return false;
    for( i = 0; i < INDEX.size(); i++)
      if (((Integer)maskt.get( i)).intValue() != -1)
        if (((Integer)maskt.get( i)).intValue() != ((Integer)INDEX.get( i)).intValue())
        return false;
    }//try
  catch (Exception e) {
    System.out.println( ERROR + "selMaskt. " + "i:" + i);
    System.out.println( maskt);
    System.out.println( INDEX);
    e.printStackTrace();
    System.exit(0);
  }//catch
  
  return true;
}//selMaskt

/**
* Proyeccion de medidas.
* @return -1: pasa true, x: no pasa false
**/
public static boolean prjMaskr( int k) {

  if ( projection)
     return (((Double)maskr.get( k)).doubleValue() == -1.0);
     else return true;
}//prjMaskr

/**
* Proyeccion de respuesta.
* @return -1: pasa true, x: no pasa false
**/
public static boolean prjMasks( int k) {

  if ( projection)
     return (((Integer)masks.get( k)).intValue() == -1);
     else return true;
}//prjMasks

/**
* Proyeccion de attributos.
* @return -1: pasa true, x: no pasa false
**/
public static boolean prjMaskt( int k) {

  if ( projection)
     return (((Integer)maskt.get( k)).intValue() == -1);
     else return true;
}//prjMaskt

private void ver( String msg) {

  System.out.println( "ver mask: " + msg);
  System.out.println( wr);
  System.out.println( maskr);
  System.out.println( ws);
  System.out.println( masks);
  System.out.println( wt);
  System.out.println( maskt);
  System.out.println( wend);

  return;
}//ver

}//class maskTable30
