//BASE30//
package target;

//
import java.lang.Object.*;
import java.util.*;
import java.lang.*;
import java.io.*;
import java.math.*;

//
import kernel.*;
import control.*;
import kbm2l.*;
import tables.*;

/**
* Clase base30, cambio de base los indices. Definicin.
* La base de esquema es BC, canonica. Estructura fisica de la lista.
* La transposiciones de indices comprueban el dominio, se permiten indices kbm2l.unKB (-1)
* Las bases son vectores estandar. [DOM-BAS-ATT-ROL-IMP]
* Parte principal:
*  6 -        720.
*  7 -      5,040.
*  8 -     40,320.
*  9 -    362,880.
* 10 -   3,628,800.
* 11 -  39,916,800.
* 12 - 479,001,600.
* @see Vector
* @author Juan A. Fdez del Pozo de Salamanca
* @version 3.0
**/
public  class  base30  extends  basex30  {

  private static final String ERROR = "Error(base30): ";

/**
* Contructor por defecto.
* [].
**/
public base30( ) {

  DOM = new Vector();//2 2 2 2 2
  BAS = new Vector();//0 1 2 3 4
  ATT = new Vector();//0 1; 0 1; 0 1; 0 1; 0 1; 
  ROL = new Vector();//-1-1-1-1-1
  IMP = new Vector();//-1-1-1-1-1
}//base30

/**
* Contructor por defecto.
* [].
**/
public base30( int t) {

  DOM = new Vector( t);//2 2 2 2 2
  BAS = new Vector( t);//0 1 2 3 4
  ATT = new Vector( t);//0 1; 0 1; 0 1; 0 1; 0 1; 
  ROL = new Vector( t);//-1-1-1-1-1
  IMP = new Vector( t);//-1-1-1-1-1
}//base30

/**
* Contructores de una base canonica.
* [(d0,0,V)(d1,1,V)(d2,2,V)....(dt-12,t-1,D)]. ROL DecTab o UtiTab.
* @param t dimension de la base
* @param D dominio
* @param d numero de decisiones d < t 
* @param rolV label del rol de variables 
* @param rolD label del rol de decisiones
**/
public base30( int t, Vector D, int d, Integer rolV, Integer rolD) {

  if ( D != null) {
    DOM = index30.cpT( D);
    } else  {//por defecto es binario
    DOM = new Vector();
    for( int i = 0; i < t; i++) {
      DOM.add( new Integer( 2));
      }//for i
    }//if D  

  BAS = new Vector();
  for( int i = 0; i < t; i++) {
    BAS.add( new Integer( i));
    }//for i

  ATT = new Vector();
  for( int i = 0; i < t; i++) {
    Vector A = new Vector();
    int e = ((Integer)DOM.get( i)).intValue();
    for( int j = 0; j < e; j++) {
      A.add( new Integer( j));      
      }//for j
    ATT.add( A);  
    }//for i
  
  ROL = new Vector();
  for( int i = 0; i < t-d; i++) {
    ROL.add( rolV);//variables
    }//for i

  for( int i = t-d; i < t; i++) {
    ROL.add( rolD);//decisiones
    }//for i

  IMP = new Vector();
  for( int i = 0; i < t; i++) {
    IMP.add( new Integer( -1));
    }//for i
 
}//base30

/**
* Contructores de una base canonica.
* [(d0,0,X)(d1,1,Y)(d2,2,Y)....(dt-1,t-1,P)]. ROL CPT.
* @param t tamao de la base
* @param D dominio.
* @param rolX variable 
* @param rolY contexto
* @param rolP probab
**/
public base30( int t, int MaxD, Vector D, Integer rolX, Integer rolY, Integer rolP) {
  
  if ( D != null) {
    DOM = index30.cpT( D);
    } else{
    DOM = new Vector();
      int maxd = 2;
      if (MaxD > maxd) maxd = MaxD;
      for( int i = 0; i < t; i++) {
        DOM.add( new Integer( maxd));
        }//for i      
    }//if D

  BAS = new Vector();
  for( int i = 0; i < t; i++) {
    BAS.add( new Integer( i));
    }//for i

  ATT = new Vector();
  for( int i = 0; i < t; i++) {
    Vector A = new Vector();
    int e = ((Integer)DOM.get( i)).intValue();
    for( int j = 0; j < e; j++) {
      A.add( new Integer( j));      
      }//for j
    ATT.add( A);  
    }//for i

  ROL = new Vector();
  ROL.add( rolX);    
  for( int i = 1; i < t-1; i++) {
    ROL.add( rolY);
    }//for i
  ROL.add( rolP);
 
  IMP = new Vector();
  for( int i = 0; i < t; i++) {
    IMP.add( new Integer( -1));
    }//for i

}//base30

/**
* Contructores de una base.
* (null, null, null) -> homogenea, canonica y sin role.
* [(d0,b0,-1)(d1,b1,-1)(d2,b2,-1)....(dt-1,bt-1,-1)]. sin ROL.
* @param t tamao de la base
* @param MaxD si D es null, cardinales homogeneos por defecto.
* @param D dominio
* @param B base 
* @param R rol
**/
public base30( int t, int MaxD, Vector D, Vector B, Vector R) {
  
    if ( D != null) {
      DOM = index30.cpT( D);
      } else{
      DOM = new Vector();
      int maxd = 2;
      if (MaxD > maxd) maxd = MaxD;
      for( int i = 0; i < t; i++) {
        DOM.add( new Integer( maxd));
        }//for i      
      }//if D

    if ( B != null) {
      BAS = index30.cpT( B);
      } else {
      BAS = new Vector();
      for( int i = 0; i < t; i++) {
        BAS.add( new Integer( i));
        }//for i      
      }//if B      

  ATT = new Vector();
  for( int i = 0; i < t; i++) {
    Vector A = new Vector();
    int e = ((Integer)DOM.get( i)).intValue();
    for( int j = 0; j < e; j++) {
      A.add( new Integer( j));      
      }//for j
    ATT.add( A);  
    }//for i

    if ( R != null) {
      ROL = index30.cpT( R);    
      } else {
      ROL = new Vector();
      for( int i = 0; i < t; i++) {
        ROL.add( new Integer( index30.unKB));
        }//for i
      }//if R 

  IMP = new Vector();
  for( int i = 0; i < t; i++) {
    IMP.add( new Integer( -1));
    }//for i

  if (!(( t == DOM.size()) && ( t == BAS.size()) && ( t == ROL.size()))) {
     System.out.println(ERROR + "base30().");
     new Exception().printStackTrace();
     System.out.println("DOM:"+DOM.size());
     System.out.println("BAS:"+BAS.size());
     for( int i = 0; i < size(); i++)
       System.out.println("BAS:"+att( i).size());
     System.out.println("ROL:"+ROL.size());
     System.out.println("IMP:"+IMP.size());
     exit( 0);
  }//
 
}//base30

/**
* Constructor copia de bases
* @param C la base a copiar
**/
public base30( base30 C) {

  this.DOM = index30.cpT( C.Dom());
  this.BAS = index30.cpT( C.Bas());
  this.ATT = new Vector();//C.ATT;
  for( int i = 0; i < BAS.size(); i++) {
     this.ATT.add( i, index30.cpT( C.att( i)));     
     }
  this.ROL = index30.cpT( C.Rol());
  this.IMP = index30.cpT( C.Imp());

}//cpB

/**
* DOM - Att's
**/
public Vector Dom( ) {

  return (DOM);
}//Dom

/**
* BAS - Att's
* Vector del orden del esquema de atributos
**/
public Vector Bas( ) {

  return (BAS);
}//Bas

/**
* ATT - Att's
* Vector de vectores de orden de dominio
**/
public Vector Att( ) {

  return (ATT);
}//Att

/**
* ROL - Att's
* Vector de rol de los atributos en el esquema
**/
public Vector Rol( ) {

  return (ROL);
}//Rol

/**
* IMP - Att's
* Vector de etiquetas de importancia de los atributos en la lista
**/
public Vector Imp( ) {

  return (IMP);
}//Imp

/**
* DOM - Att's
* inicia DOM index30.unKB
* @param t cardinal del esquema
**/
public Vector Dom( int t) {

  DOM = new Vector();
  for( int i = 0; i < t; i++) {
    DOM.add( new Integer( index30.unKB));
    }//for i  
  
  return (DOM);
}//Dom

/**
* BAS - Att's
* inicia BAS index30.unKB
* @param t cardinal del esquema
**/
public Vector Bas( int t) {

  BAS = new Vector();
  for( int i = 0; i < t; i++) {
    BAS.add( new Integer( index30.unKB));
    }//for i  
  
  return (BAS);
}//Bas

/**
* ATT - Att's
* inicia ATT index30.unKB
* @param t cardinal del esquema
**/
public Vector Att( int t) {

  ATT = new Vector();
  for( int i = 0; i < t; i++) {
    Vector A = new Vector();
    for( int j = 0; j < dom( i); j++) {
      A.add( new Integer( index30.unKB));
      }//for j  
    ATT.add( A);
    }//for i  
  
  return (ATT);
}//Att

/**
* ROL - Att's
* inicia ROL index30.unKB
* @param t cardinal del esquema
**/
public Vector Rol( int t) {

  ROL = new Vector();
  for( int i = 0; i < t; i++) {
    ROL.add( new Integer( index30.unKB));
    }//for i  
  
  return (ROL);
}//Rol

/**
* IMP - Att's
* inicia IMP index30.unKB
* @param t cardinal del esquema
**/
public Vector Imp( int t) {

  IMP = new Vector();
  for( int i = 0; i < t; i++) {
    IMP.add( new Integer( index30.unKB));
    }//for i  
  
  return (IMP);
}//Imp

/**
* get DOM(i)
* @param i dim-att
**/
public int dom( int i) {

  return ((Integer)DOM.get( i)).intValue();
}//dom

/**
* get BAS(i)
* @param i dim-att
**/
public int bas( int i) {

  return ((Integer)BAS.get( i)).intValue();
}//bas

/**
* get ATT(i)
* Vector de orden de los elementos del domnio del atributo del esquema
* que tiene la posicin (i) en la base, es una permutacion.
* @param i dim-att
**/
public Vector att( int i) {

  return ((Vector)ATT.get( i));
}//att

/**
* get ATT(i,j)
* Valor de dominio que ocupa la posicin (j) del atributo de la base en la posicin (i) 
* @param i dim-att
* @param j dim-valor
**/
public int att( int i, int j) {

  return ((Integer)att( i).get( j)).intValue();
}//att

/**
* get ROL(i)
**/
public int rol( int i) {

  return ((Integer)ROL.get( i)).intValue();
}//rol

/**
* get IMP(i)
**/
public int imp( int i) {

  return ((Integer)IMP.get( i)).intValue();
}//imp

/**
* set DOM(i,j)
**/
public int dom( int i, int j) {

  DOM.set( i, new Integer( j));
  return ((Integer)DOM.get( i)).intValue();
}//dom

/**
* set BAS(i,j)
**/
public int bas( int i, int j) {

  BAS.set( i, new Integer( j));
  return ((Integer)BAS.get( i)).intValue();
}//bas

/**
* set ATT(i,V)
**/
public Vector att( int i, Vector V) {

  ATT.set( i, V);
  return ((Vector)ATT.get( i));
}//att

/**
* set ATT(i,j,v)
* Asignacin del valor (v) de dominio a la posicin (j) del atributo de la base en la posicin (i) 
* @param i dim-att
* @param j dim-valor
* @param v valor
**/
public int att( int i, int j, int v) {

  ((Vector)ATT.get( i)).set( j, new Integer( v));
  return v;
}//att

/**
* set ROL(i)
**/
public int rol( int i, int j) {

  ROL.set( i, new Integer( j));
  return ((Integer)ROL.get( i)).intValue();
}//rol

/**
* set IMP(i)
**/
public int imp( int i, int j) {

  IMP.set( i, new Integer( j));
  return ((Integer)IMP.get( i)).intValue();
}//imp

/**
* indexOf
**/
public int indexOfdom( int i) {

  return DOM.indexOf( new Integer( i));
}//indexOfdom

/**
* indexOf
**/
public int indexOfbas( int i) {

  return BAS.indexOf( new Integer( i));
}//indexOfbas

/**
* indexOf
**/
public int indexOfatt( Vector V) {

  return ATT.indexOf( V);
}//indexOfatt

/**
* indexOf
**/
public int indexOfatt( int k, int i) {

  return ((Vector)ATT.get( k)).indexOf( new Integer( i));
}//indexOfatt

/**
* indexOf
**/
public int indexOfrol( int i) {

  return ROL.indexOf( new Integer( i));
}//indexOfrol

/**
* indexOf
**/
public int indexOfimp( int i) {

  return IMP.indexOf( new Integer( i));
}//indexOfimp

/**
* add(d, b, r)
* Aade una componente de la base al final de la estructura.
* El cardinal del dominio (d)->(DOM),
* el orden relativo a la base canonica (b)->(BAS) y 
* la etiqueta del rol (r)->(ROL).
* El orden del dominio de los atributos es canonico, (0,1,...,d-1)->(ATT).
* La etiqueta de importancia es desconocida (index30.unKB)->(IMP).
**/
public void add( int d, int b, int r) {

  DOM.add( new Integer( d));
  BAS.add( new Integer( b));
  Vector Att = new Vector();
  for(int i = 0; i < d; i++)
    Att.add( new Integer( i));
  ATT.add( Att);
  ROL.add( new Integer( r));
  IMP.add( new Integer( index30.unKB));
  return;
}//add

/**
* add(i, d, b, r)
* Aade una componente de la base en la posicin (i) de la estructura.
* El cardinal del dominio (d)->(DOM),
* el orden relativo a la base canonica (b)->(BAS) y 
* la etiqueta del rol (r)->(ROL).
* El orden del dominio de los atributos es canonico, (0,1,...,d-1)->(ATT).
* La etiqueta de importancia es desconocida (index30.unKB)->(IMP).
**/
public void add( int i, int d, int b, int r) {

  DOM.add( i, new Integer( d));
  BAS.add( i, new Integer( b));
  Vector Att = new Vector();
  for(int j = 0; j < d; j++)
    Att.add( new Integer( j));
  ATT.add( i, Att);
  ROL.add( i, new Integer( r));
  IMP.add( i, new Integer( index30.unKB));
  return;
}//add

/**
* add(d, b, Att, r)
* Aade una componente de la base al final de la estructura.
* El cardinal del dominio (d)->(DOM),
* el orden relativo a la base canonica (b)->(BAS) y 
* la etiqueta del rol (r)->(ROL).
* El orden del dominio de los atributos no es canonico, (Att)->(ATT).
* La etiqueta de importancia es desconocida (index30.unKB)->(IMP).
**/
public void add( int d, int b, Vector Att, int r) {

  DOM.add( new Integer( d));
  BAS.add( new Integer( b));
  ATT.add( Att);
  ROL.add( new Integer( r));
  IMP.add( new Integer( index30.unKB));
  return;
}//add

/**
* add(i, d, b, Att, r)
* Aade una componente de la base en la posicin (i) de la estructura.
* El cardinal del dominio (d)->(DOM),
* el orden relativo a la base canonica (b)->(BAS) y 
* la etiqueta del rol (r)->(ROL).
* El orden del dominio de los atributos es canonico, (Att)->(ATT).
* La etiqueta de importancia es desconocida (index30.unKB)->(IMP).
**/
public void add( int i, int d, int b, Vector Att, int r) {

  DOM.add( i, new Integer( d));
  BAS.add( i, new Integer( b));
  ATT.add( i, Att);
  ROL.add( i, new Integer( r));
  IMP.add( i, new Integer( index30.unKB));
  return;
}//add

/**
* remove(i)
**/
public void remove( int i) {

  DOM.remove( i);
  BAS.remove( i);
  ATT.remove( i);
  ROL.remove( i);
  IMP.remove( i);
  return;
}//remove

/**
* DOM+BAS+ATT+ROL+IMP
**/
public String toString( ) {

  return ("\n DOMAIN: " + DOM.toString() + 
          "\n BASE:   " + BAS.toString() + 
          "\n ATTRIB: " + ATT.toString()+
          "\n ROL:    " + ROL.toString()+
          "\n IMPORT: " + IMP.toString());
}//toString

/**
* DOM+BAS+ATT+ROL+IMP
**/
public String toString( lista30 l30) {

  String att = new String();
  try {
	  
  for(int i = 0; i < size(); i++)
    if ( l30.Esqu.numDecisiones == 0)
      att = att + ((Vector)ATT.get( bas( i))).toString() + "\n";
      else att = att + i + "::" + ((Vector)ATT.get( i)).toString() + ": " + 
        ((String)l30.Esqu.Atributos.get( bas( i))) + "::" + bas( i) + "\n";

  } catch (Exception e) { 
    System.out.println( ERROR + "toString( lista30 l30).0");
    System.out.println( ATT.toString());
    System.out.println( l30.Esqu.Atributos.toString());
    f30.exit( 0, e);
  }//catch

  if ( l30.checkMode)
    return (DOM.toString() + "\n" + BAS.toString() + "\n" + att  + 
            ROL.toString() + "\n" + IMP.toString());
    else 
      return (DOM.toString() + "\n" + BAS.toString() + "\n" + att);
}//toString

/**
* complejidad de la base 
**/
public double complex() {

  long bsz = BAS.size();
  double c = dato30.logFactorial( bsz);
 	
  return c;	
}//complex
  
/**
* complejidad de la base extendida
**/
public double xComplex() {

  if ( XCOMPLEX < 0) {
    long bsz = BAS.size();
    double c = dato30.logFactorial( bsz);
    for( int i = 0; i < bsz; i++)	
      c = c + dato30.logFactorial( (long)dom( i));
      XCOMPLEX = c;
    }
   
  return XCOMPLEX;	
}//xComplex

}//class base30