//TARGET30//
package target;

//
import java.lang.Object.*;
import java.util.*;
import java.lang.*;
import java.io.*;

//
import kernel.*;
import kbm2l.*;
import control.*;

/**
* Clase target30
* Bases de las listas target para las copias std y fast
* de sintesis H2, Cd, Ci, HR, 2Opt, y Genetica.
* Utiliza los medotos swapbase30 para cambiar de base las listas target.
* No hay datos miembro y todos los metodos son static.
* Implementa diferentes tipos de permutaciones o 
* cambios de base para las listas.
* Las bases son vectores estandar.
* @author Juan A. Fdez del Pozo de Salamanca
* @version 3.0
**/

public  class  target30  extends  base30  {

  private static final String ERROR = "Error(target30): ";

/**
* Constructor por defecto.
* Sin Parametros.
**/
public target30( ) {

}//target30

/**
* Genera una base alineada con el vector orden (REQUEST)
* @param b30 base de la l30, lista objeto del cambio de base
* @param orden Vector con el orden de los atributos
* @return base alineada.
**/
public  static  base30  swap( base30 b30, Vector orden) {

  base30 base = new base30( b30);//la base original
  int i = 0, j = 0;

  try {

  for( i = 0; i < b30.size(); i++) {
    Integer O = (Integer)(orden.get( i));
    int o = O.intValue();
    if ( o != base.bas( i)) {
      j = (base.Bas()).indexOf( O);

      int d0 = base.dom( i); int d1 = base.dom( j);//dom      
      base.dom( i,  d1);     base.dom( j, d0);

      int b0 = base.bas( i); int b1 = base.bas( j);//bas      
      base.bas( i, b1);      base.bas( j, b0);

      Vector V0 = base.att( i); Vector V1 = base.att( j);//      
      base.att( i, V1);         base.att( j, V0);

      int r0 = base.rol( i); int r1 = base.rol( j);//rol      
      base.rol( i, r1);      base.rol( j, r0);
        
      int s0 = base.imp( i); int s1 = base.imp( j);//rol      
      base.imp( i, s1);      base.imp( j, s0);

      }//if
    }//for

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "swap");
    System.out.println( "i: " + i + "  j: " + j );
    System.out.println( "oldBase: " + b30.toString());
    System.out.println( "newBase: " + base);
    frame30.exit( 0, e);
  }//catch

  if ( !base30.cmpB( base, b30)) {
    System.out.println( ERROR + "swap cmpB");
    System.out.println( "oldBase: " + b30.toString());
    System.out.println( "newBase: " + base.toString( ));
    frame30.exit( 0, new Exception());  
   }

  return base;
}////swap

/**
* Cambio de base Hamming-2.
* Comprueba si las posiciones que permutan con validas.
* Permuta los valores (i,j) de dominio y base (role) de la lista.
* Es utilizada por otras formas de permutacion.
* @see kbm2l30#dimension()
* @see base30#dom(int)
* @see base30#dom(int,int)
* @see base30#bas(int)
* @see base30#bas(int,int)
* @see metrica30#H(base30,base30)
* @param b30 base de la l30, lista objeto del cambio de base
* @param i posicion del atributo de la base que cambia
* @param j posicion del atributo de la base que cambia
* @return b30 la base nueva 
**/
public  static  base30  swap2( base30 b30, int i, int j) {

  if (( i < 0) || ( i >= b30.size())) {
    System.out.println( ERROR + "swap2: i=" + i); frame30.exit(0);
    }
  if (( j < 0) || ( i >= b30.size())) {
    System.out.println( ERROR + "swap2: j=" + j); frame30.exit(0);
    }

  base30 oldBase = new base30( b30);

  try {//intercambia i,j

    int d0 = b30.dom( i); int d1 = b30.dom( j);//dom    
    b30.dom( i, d1);      b30.dom( j, d0);//swap

    int b0 = b30.bas( i); int b1 = b30.bas( j);//bas    
    b30.bas( i, b1);      b30.bas( j, b0);//swap

    Vector V0 = b30.att( i); Vector V1 = b30.att( j);//att
    b30.att( i, V1);         b30.att( j, V0);//swap

    int r0 = b30.rol( i); int r1 = b30.rol( j);//rol    
    b30.rol( i, r1);      b30.rol( j, r0);//swap

    int s0 = b30.imp( i); int s1 = b30.imp( j);//imp    
    b30.imp( i, s1);      b30.imp( j, s0);//swap

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "swap2");
    System.out.println( "i: " + i + "  j: " + j );
    System.out.println( "oldBase: " + oldBase.toString());
    System.out.println( "newBase: " + b30.toString());
    frame30.exit( 0, e);
  }//catch

  if ( metrica30.H( b30, oldBase) > 2) {
    System.out.println( ERROR + "swap2: H!=" + 2 + "   i:" + i + "   j:" + j);
    System.out.println( b30.Bas());
    System.out.println( oldBase.Bas());
    frame30.exit( 0);
    }
    
  if ( !base30.cmpB( oldBase, b30)) {
    System.out.println( ERROR + "swap2 cmpB");
    System.out.println( "oldBase: " + oldBase.toString( ));
    System.out.println( "newBase: " + b30.toString( ));
    frame30.exit( 0, new Exception());  
   }

  return b30;  
}//swap2

/**
* Cambio de base Cd --->>>, circular a la derecha: 
* i es 0, j es 2. La distancia Hamming es n-i. j > 0 y j < n-i
* 01234....n --->>> 1234....n0, primer paso, 1234....n0 --->>> 234....n01, segundo paso
* 01234....n
* 234....n01
* Comprueba si las posiciones que permutan con validas: 0..i..n
* fija el numero de pasos j al modulo del subcojunto i..n.
* Permuta los valores (i) a la posicion menor del dominio y base de la lista.
* @see kbm2l30#dimension()
* @see base30#dom(int)
* @see base30#dom(int,int)
* @see base30#bas(int)
* @see base30#bas(int,int)
* @see metrica30#H(base30,base30)
* @param l30 lista objeto del cambio de base
* @param i posicion del atributo de la base que cambia mas a la izq.
* @param j numero de pasos circulares a la derecha
* @return la base nueva 
**/
protected  static  base30  swapCd( lista30 l30, int i, int j) {

  if (( i < 0) || ( i >= l30.dimension())) {
    System.out.println( ERROR + "swapCd: i=" + i);
    System.exit( 0);
    }
    
  if ( j < 0) {
    System.out.println( ERROR + "swapCd: j=" + j);
    System.exit( 0);
    }

  if ( i == ( l30.dimension()-1)) {
    return l30.Base;
    }
    
  if ( j == 0) {
    return l30.Base;
    }

  base30 oldBase = new base30( l30.Base);
//  j = j % (l30.dimension() - i);

  try {
    for ( int k = 0; k < j; k++) {//copia circular parcial derecha
      // el i al final y borrar el i
      l30.Base.add( l30.Base.dom( i), l30.Base.bas( i), l30.Base.rol( i));
      l30.Base.remove( i);       
      }//for

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "swapCd");
    System.out.println( "i: " + i + "  j: " + j );
    System.out.println( "oldBase: " + oldBase.toString());
    System.out.println( "newBase: " + l30.Base.toString());
    frame30.exit( 0, e);
  }//catch

  int h = metrica30.H( l30.Base, oldBase);
  if ( h != 0)
  if ( h != (l30.dimension()-i)) {
    System.out.println( "i: " + i + "  j: " + j + "  h: " + h);
    System.out.println( "oldBase: " + oldBase.toString());
    System.out.println( "newBase: " + l30.Base.toString());
    System.out.println( ERROR + "swapCd: H!=" + (l30.dimension()-i));
    frame30.exit( 0);
    }

  return l30.Base;  
}//swapCd

/**
* Cambio de base Ci <<<---, circular a la izquierda: 
* i es n, j es 2. La distancia Hamming es i.
* 01234....n --->>> n01234....(n-1), primer paso
* n01234....(n-1) --->>> (n-1)n01234....(n-2), segundo paso
* Comprueba si las posiciones que permutan con validas: 0..i..n
* fija el numero de pasos j al modulo del subcojunto 0..i.
* Permuta los valores (i) a la posicion mayor del dominio y base de la lista.
* @see kbm2l30#dimension()
* @see base30#dom(int)
* @see base30#dom(int,int)
* @see base30#bas(int)
* @see base30#bas(int,int)
* @see metrica30#H(base30, base30)
* @param l30 lista objeto del cambio de base
* @param i posicion del atributo de la base que cambia mas a la derecha.
* @param j numero de pasos circulares a la izquierda
* @return la base nueva 
**/
protected  static  base30  swapCi( lista30 l30, int i, int j) {

  if (( i < 0) || ( i >= l30.dimension())) {
    System.out.println( ERROR + "swapCi: i=" + i);
    System.exit( 0);
    return null;
    }
    
  if ( j < 0) {
    System.out.println( ERROR + "swapCi: j=" + j);
    System.exit( 0);
    return null;
    }

  if ( i == 0) {
    return l30.Base;
    }
    
  if ( j == 0) {
    return l30.Base;
    }
  
  base30 oldBase = new base30( l30.Base);
//  j = j % i;

  try {
    for ( int k = 0; k < j; k++) {//copia circular parcial izquierda
      // el i al 0 y borrar el i+1, anterior i
      l30.Base.add( 0, l30.Base.dom( i), l30.Base.bas( i), l30.Base.rol( i));    
      l30.Base.remove( i+1);
      }//for

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "swapCi");
    System.out.println( "i: " + i + "  j: " + j );
    System.out.println( "oldBase: " + oldBase.toString());
    System.out.println( "newBase: " + l30.Base.toString());
    frame30.exit( 0, e);
  }//catch

  if ( metrica30.H( l30.Base, oldBase) != (i+1)) {
    System.out.println( "i: " + i + "  j: " + j);
    System.out.println( "oldBase: " + oldBase.toString());
    System.out.println( "newBase: " + l30.Base.toString());
    System.out.println( ERROR + "swapCi: H!=" + (i+1));
    frame30.exit( 0);
    }

  return l30.Base;  
}//swapCi

/**
* Mascara de boolean
* @param MASK vector de boolean
* @param i posicion del vector MASK
* @return true si la posicion i de MASK es true
**/
private static boolean mask( Vector MASK, int i) {

  if ( MASK == null) return false;
  return ((Boolean)MASK.get( i)).booleanValue();
}//mask

/**
* Cambio de base R, random n cambios Swap2 sobre posiciones aleatorias
* La distancia Hamming es aleatoria antre 1 y n.
* Permuta los valores (i,j) generados al azar.
* Permite cambio de base extendida.
* @see kbm2l30#dimension()
* @see base30#bas(int)
* @see target30#swap2(base30,int,int)
* @param l30 lista objeto del cambio de base
* @return lista30#getBase() vector de la base nueva 
**/
protected  static  base30  swapR1( lista30 l30) {

//  Random gr = new Random(1227);
  Random gr = control30.gr;
  boolean x = new controlPrefs30().nBase();
  for( int m = 0; m < l30.dimension(); m++) {
    int i = 0, j = 0;
    base30 oldBase = l30.Base;
    while ( base30.eqB( oldBase, l30.Base)) {
      i = Math.abs( gr.nextInt() % l30.dimension());
      j = Math.abs( gr.nextInt() % l30.dimension());
      }//while
 
    if ( x) {
      int di1 = Math.abs( control30.gr.nextInt() % l30.Base.dom( i));
      int di2 = Math.abs( control30.gr.nextInt() % l30.Base.dom( i));
      l30.Base = xswap2( l30.Base, i, di1, di2);
        
      int dj1 = Math.abs( control30.gr.nextInt() % l30.Base.dom( j));
      int dj2 = Math.abs( control30.gr.nextInt() % l30.Base.dom( j));
      l30.Base = xswap2( l30.Base, j, dj1, dj2);
      }
      
    l30.Base = swap2( l30.Base, i, j);//dom.bas.att.rol.imp.
  }//for
  
  return l30.Base;  
}//swapR1

/**
* Cambio de base R, random n cambios Swap2 sobre posiciones aleatorias
* La distancia Hamming es aleatoria antre 0 y n.
* Permuta los valores (i,j) generados al azar.
* Azar pero distinta de cierto conjunto de bases.
* Azar pero a cierta distancia max/min.
* Azar pero fijando ciertos atributos mask (vector {false,true}): MASK.
* @see kbm2l30#dimension()
* @see base30#dom(int)
* @see base30#dom(int,int)
* @see base30#bas(int)
* @see base30#bas(int,int)
* @see metrica30#H(base30, base30)
* @param b30 base de la l30, lista objeto del cambio de base
* @param BasesList historico de la busqueda.
* @param Dmin la distancia entre la base inicial y resultado debe ser mayor o igual que el minimo
* @param Dmax la distancia entre la base inicial y resultado debe ser menor o igual que el maximo
* @param MASK posiciones fijas true, si es null valen todos los movs.
* @return la base nueva o null si no es factible generar la base pedida.
**/
protected  static  base30  swapR( base30 b30, boolean xb, Vector BasesList, int Dmin, int Dmax, Vector MASK) {

  long loops = 0;
  double g = -1; int i = 0, j = 0, bsz = b30.size();
  base30 oldBase = new base30( b30);//copia  

  try {
	  
  do {
    do {  
      for( int m = 0; m < bsz; m++) {
        i = 0; j = 0;

        try {
        while ( i == j) {        
          i = Math.abs( control30.gr.nextInt() % bsz);
          j = Math.abs( control30.gr.nextInt() % bsz);
          if ( !mask( MASK, i) || !mask( MASK, j)) i = j;
          if ( loops < 10 * bsz) loops++; else break;//return null;
          }//while
        }//try
        catch (Exception e) {
          System.out.println( ERROR + "swapR.1"); frame30.exit( 0, e);
        }//catch

        try {
        if ( xb) {
          int di1 = Math.abs( control30.gr.nextInt() % b30.dom( i)); 
          int di2 = Math.abs( control30.gr.nextInt() % b30.dom( i)); 
          if ( di1 != di2)
            b30 = xswap2( b30, i, di1, di2);
        
          int dj1 = Math.abs( control30.gr.nextInt() % b30.dom( j));
          int dj2 = Math.abs( control30.gr.nextInt() % b30.dom( j));
          if ( dj1 != dj2)
            b30 = xswap2( b30, j, dj1, dj2);
          }
        }//try
        catch (Exception e) {
          System.out.println( ERROR + "swapR.2"); frame30.exit( 0, e);
        }//catch

         b30 = swap2( b30, i, j);// i != j //dom.bas.att.rol.imp.
      }//for m
      
      if ( loops < 4 * bsz) loops++; else break;//return null;      
    } while( BasesList.indexOf( b30) >= 0);//est en la lista de bases

    if ( loops < 8 * bsz) loops++; else break;//return null;
    g = metrica30.G( oldBase, b30);

  } while( ( g < Dmin) || ( g > Dmax));//distancia

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "swapR");
    System.out.println( "i: " + i + "  j: " + j );
    System.out.println( "oldBase: " + oldBase.Bas( ));
    System.out.println( "newBase: " + b30.toString( )); frame30.exit( 0, e);
  }//catch
 
  return b30;  
}//swapR

/**
* Cambio de base S, palindroma
* @param l30 lista objeto del cambio de base
**/
protected  static  base30  swapS( base30 b30) {

    base30.check( b30);
    base30 B = new base30( b30);
    base30 S = new base30();//palindromas    
    for ( int i = b30.size()-1; i >= 0; i--) {      
      S.add( B.dom( i), B.bas( i), B.att( i), B.rol( i));
      }//for

  return S;//cambia la base del argumento    
}//swapS

/**
* Cambio de base SuperS, palindroma super-simetrica
* @param l30 lista objeto del cambio de base
**/
protected  static  base30  swapSS( lista30 l30) {

    base30 B = l30.Base;
    base30 SS = new base30();//palindromas
    for ( int i = l30.ParIndice()-1; i >= 0; i--) {      
      SS.add( B.dom( i), B.bas( i), B.rol( i));
      }//for

    for ( int i = l30.ParIndice()-1; i >= 0; i--) {  
      Vector Att = new Vector();
      for(int j = B.dom( i)-1; j >= 0; j--)
        Att.add( new Integer( j));
      SS.att( i, Att);
      }//for

    l30.Base = SS;//cambia la base del argumento

  return l30.Base;    
}//swapSS

/**
* Cambio de base G, cambio a la base newBase.
* La distancia Hamming es la que haya entre la base original y la nueva.
* comprueba que la bases orginal y nueva sean compatibles
* Permuta los valores de dominio acorde a la base nueva.
* @see base30#size()
* @param l30 lista objeto del cambio de base
* @param newBase base nueva para lista objeto del cambio de base
* @return la base nueva 
**/
protected  static  base30  swapB( lista30 l30, base30 newBase) {

  if ( newBase.size() != l30.Base.size()) {
    System.out.println( ERROR + "swapB.1");
    System.out.println( "newBase.size()= " + newBase.size());
    System.out.println( "newBase= " + newBase.Bas( ));
    System.out.println( "oldBase.size()= " + l30.Base.size());
    System.out.println( "oldBase.size()= " + l30.Base.toString( l30));
    frame30.exit( 0);
    }

  l30.Base = new base30( newBase);//cambia la base del argumento

  return l30.Base;    
}//swapB

/**
* Cambio de base Hamming-2Opt.
* Comprueba si las posiciones que permutan con validas.
* Permuta los valores (i,j) de dominio y base de la lista.
* @see kbm2l30#dimension()
* @see base30#dom(int)
* @see base30#dom(int,int)
* @see base30#bas(int)
* @see base30#bas(int,int)
* @see metrica30#H(base30,base30)
* @param l30 lista objeto del cambio de base
* @param i posicion del atributo de la base que cambia mas a la derecha
* @param j posicion del atributo de la base que cambia ma a la izquierda
* @return la base nueva 
**/
protected  static  base30  swap2Opt( lista30 l30, int i, int j) {

  if (( i < 0) || ( i >= l30.dimension())) {
    System.out.println( ERROR + "swap2Opt: i=" + i);
    System.exit(0);
    }
  if (( j < 0)||( i >= l30.dimension())) {
    System.out.println( ERROR + "swap2Opt: j=" + j);
    System.exit(0);
    }

  base30 oldBase = new base30( l30.Base);

  try {

    for( int x = i, y = j; x < y; x++, y--) {
      int d0 = l30.Base.dom( x);
      int d1 = l30.Base.dom( y);
      l30.Base.dom( x,  d1);
      l30.Base.dom( y, d0);

      int b0 = l30.Base.bas( x);
      int b1 = l30.Base.bas( y);
      l30.Base.bas( x, b1);
      l30.Base.bas( y, b0);

      Vector V0 = l30.Base.att( i);//
      Vector V1 = l30.Base.att( j);
      l30.Base.att( i, V1);
      l30.Base.att( j, V0);

      int r0 = l30.Base.rol( x);
      int r1 = l30.Base.rol( y);
      l30.Base.rol( x, r1);
      l30.Base.rol( y, r0);

      int s0 = l30.Base.imp( x);
      int s1 = l30.Base.imp( y);
      l30.Base.imp( x, s1);
      l30.Base.imp( y, s0);

    }//for x,y

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "swap2Opt");
    System.out.println( "i: " + i + "  j: " + j );
    System.out.println( "oldBase: " + oldBase.Bas( ));
    System.out.println( "newBase: " + l30.Base.toString( l30));
    frame30.exit( 0, e);
  }//catch

  int Opt = j-i;
  if ((Opt % 2) != 0) Opt++;
  if ( metrica30.H( l30.Base, oldBase) != Opt) {
    System.out.println( ERROR + "swap2Opt: H!=" + Opt);
    frame30.exit( 0);
    }
  
  return l30.Base;  
}//swap2Opt

/**
* Cambio de base Hamming-2 en dom de un att.
* Comprueba si las posiciones que permutan con validas.
* Permuta los valores (l,m) de dominio.
* Es la utilizada por otras formas de permutacin.
* @see kbm2l30#dimension()
* @param b30 base de la l30, lista objeto del cambio de base
* @param k posicion del atributo de la base que cambia su orden de dominio
* @param l posicion del valor del atributo k de la base que cambia
* @param m posicion del valor del atributo k de la base que cambia
* @return b30 la base nueva 
**/
public  static  base30  xswap2( base30 b30, int k, int l, int m) {

  if (( l < 0) || ( l >= b30.dom( k))) {
    System.out.println( ERROR + "xswap2: l=" + l);
    System.out.println( "b30.dom( k)=" + b30.dom( k) + "  k=" + k + "  l=" + l + "  m=" + m);
    frame30.exit(0);
    }

  if (( m < 0) || ( m >= b30.dom( k))) {
    System.out.println( ERROR + "xswap2: m=" + m);
    System.out.println( "b30.dom( k)=" + b30.dom( k) + "  k=" + k + "  l=" + l + "  m=" + m);
    frame30.exit(0);
    }

  try {
    //ATT
    int v = b30.att( k, l); b30.att( k, l, b30.att( k, m)); b30.att( k, m, v);
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "xswap2");
    System.out.println( "l: " + l + "  m: " + m );
    System.out.println( "newBase: " + b30.Bas( ));
    frame30.exit( 0, e);
  }//catch

  return b30;  
}//xswap2

}//class target30
