//TARGETSTD30//
package target;

//
import java.lang.Object.*;
import java.util.*;
import java.lang.*;
import java.io.*;

//
import kbm2l.*;
import kernel.*;
import control.*;

/**
* Clase targetStd30.
* Listas target para las copias std y fast de sintesis H2, Cd, Ci, HR, 2Opt y Genetica.
* Utiliza los medotos swapbase30 para cambiar de base las listas target.
* No hay datos miembro y todos los metodos son static.
* @see controlPrefs30#modalCriterion().
* @author Juan A. Fdez del Pozo de Salamanca
* @version 3.0
**/
public class targetStd30 extends target30 {

//
  private static final String ERROR = "Error(targetStd30): ";

/**
* Constructor por defecto.
* Sin Parametros.
**/
public targetStd30( ) {

}//targetStd30

/**
* Crea una lista vacia a partir de otra lista y cambia de base a H2(ij).
* Se puede utilizar como target de la copia estandar y rapida.
* @param l30 la lista de referencia.
* @param pi posicion de la base.
* @param pj posicion de la base.
* @return lista30 la lista target
**/
public  static  lista30 Target2( lista30 l30, int pi, int pj) {

  lista30 lTarget = null;
  if ( chk) frame30.kbm2lSystem.append( "Target2 \n");
  try {

    lTarget = new lista30( l30);//std-cp

    swap2( lTarget.Base, pi, pj);//Swap2

    if (( new controlPrefs30().modalCriterion()))
      lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
      else lTarget.ini( );
    
    checkList30.chk( lTarget, "TargetH2");
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "TargetH2.");
    frame30.exit( -1, e);
  }//catch

  return lTarget;
}//Target2

/**
* Crea una lista vacia a partir de otra lista y cambia de base a H2(ij,k_ij).
* Se puede utilizar como target de la copia estandar pero no en la rapida.
* xSwap2, 1; Swap2, 2
* @param l30 la lista de referencia.
* @param pi posicion de la base.
* @param pj posicion de la base.
* @return lista30 la lista target
**/
public  static  lista30 Target2( lista30 l30, int pi, int pj, int pk, int k_i, int k_j) {

  lista30 lTarget = null;
  base30 b30 = new base30( l30.Base);
  try {

    lTarget = new lista30( l30);//std-cp

    if (new controlPrefs30().nBase())
      lTarget.Base = xswap2( b30, pk, k_i, k_j);//xSwap2, 1
    if (pi != pj) lTarget.Base = swap2( b30, pi, pj);//Swap2, 2

    if (( new controlPrefs30().modalCriterion()))
      lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
      else lTarget.ini( );
    
    checkList30.chk( lTarget, "TargetH2");
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "TargetH2.");
    frame30.exit( -1, e);
  }//catch

  return lTarget;
}//Target2

/**
* Crea una lista vacia a partir de otra lista y cambia de base 
* enviado pi a la posicion de menor peso (derecha) y desplaza a la izquierda pj veces.
* Se puede utilizar como target de la copia estandar.
* @param l30 la lista de referencia.
* @param pi posicion de la base.
* @param pj desplazamientos.
* @return lista30 la lista target
**/
public  static  lista30 TargetCd( lista30 l30, int pi, int pj) {

  lista30 lTarget = null;
  try {

    lTarget = new lista30( l30);//std

    swapCd( lTarget, pi, pj);//SwapCd

    if (( new controlPrefs30().modalCriterion()))
      lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
      else lTarget.ini( );

    checkList30.chk( lTarget, "TargetCd");
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "TargetCd.");
    frame30.exit( 0, e);
  }//catch

  return lTarget;
}//TargetCd

/**
* Crea una lista vacia a partir de otra lista y cambia de base 
* enviado pi a la posicion de mayor peso (izquierda) y desplaza a la derecha pj veces.
* Se puede utilizar como target de la copia estandar.
* @param l30 la lista de referencia.
* @param pi posicion de la base.
* @param pj desplazamientos.
* @return lista30 la lista target
**/
public  static  lista30 TargetCi( lista30 l30, int pi, int pj) {

  lista30 lTarget = null;
  try {

    lTarget = new lista30( l30);//std

    swapCi( lTarget, pi, pj);//SwapCi

    if (( new controlPrefs30().modalCriterion()))
      lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
      else lTarget.ini( );

    checkList30.chk( lTarget, "TargetCi");
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "TargetCi.");
    frame30.exit( 0, e);
  }//catch

  return lTarget;
}//TargetCi

/**
* Crea una lista vacia a partir de otra lista y cambia de base Random.
* Se puede utilizar como target de la copia estandar.
* Puede restringir los atributos que permutan entre attTop y AttBot.
* @param l30 la lista de referencia.
* @return lista30 la lista target en una base nueva
**/
public  static  lista30 TargetR( lista30 l30, int attTop, int attBot) {

  lista30 lTarget = null;
  Vector MASK = new Vector();// Boolean
  boolean xb = new controlPrefs30().nBase();//extended base?

  for( int i = 0; i < l30.dimension(); i++) 
    if (( i >= attTop) && ( i <= attBot)) 
      MASK.add( new Boolean( 0 == 0)); // true
      else MASK.add( new Boolean( 0 != 0));// false

  try {
    lTarget = new lista30( l30);//std, lista vacia,

    int loops = l30.dimension();//t
    while ( loops-- > 0) {
      lTarget.Base = swapR( new base30( lTarget.Base), xb, lTarget.BasesList, 1, 2*l30.dimension(), MASK);//RandomSwap  
      if ( lTarget.Base == null) return null;
      }//while

    try {
      if (( new controlPrefs30().modalCriterion()))
        lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
        else lTarget.ini( );

      checkList30.chk( lTarget, "TargetR");
    }//try
    catch (Exception e) {
      System.out.println( ERROR + "TargetR.1");
      frame30.exit( 0, e);
    }//catch

  }//try
  catch (Exception e) {
    System.out.println( ERROR + "TargetR.");
    frame30.exit( 0, e);
  }//catch

  return lTarget;
}//TargetR

/**
* Crea una lista vacia a partir de otra lista y cambia de base a la palindroma (^-1)
* Se puede utilizar como target de la copia estandar.
* @param l30 la lista de referencia.
* @return lista30 la lista target.
**/
public  static  lista30 TargetS( lista30 l30) {

  lista30 lTarget = null;
  try {

    lTarget = new lista30( l30);//std

    lTarget.Base = swapS( lTarget.Base);

    lTarget.ini( );

    checkList30.chk( lTarget, "TargetS");
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "TargetS.");
    frame30.exit( 0, e);
  }//catch

  return lTarget;
}//TargetS

/**
* Crea una lista vacia a partir de otra lista y cambia de base newBase
* Se puede utilizar como target de la copia estandar y soporte del fitness.
* las bases son correctas y compatibles
* @param l30 la lista de referencia.
* @param newBase la nueva base de referencia.
* @return lista30 la lista target.
**/
public  static  lista30 TargetG( lista30 l30, base30 newBase) {

  lista30 lTarget = null;
  try {

    lTarget = new lista30( l30);//replica std de la estructura de l30 en lTarget
    swapB( lTarget, newBase);//base a proposito

    if (( new controlPrefs30().modalCriterion()))
      lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
      else lTarget.ini( );

    checkList30.chk( lTarget, "TargetG");
  } catch (Exception e) {
    System.out.println( ERROR + "TargetG. ");
    frame30.exit( 0, e);
  }//catch

  return lTarget;
}//TargetG

/**
* Crea una lista vacia a partir de otra lista y cambia de base a H2(ij).
* Se puede utilizar como target de la copia estandar.
* @param l30 la lista de referencia.
* @param pi posicion de la base.
* @param pj posicion de la base.
* @return lista30 la lista target.
**/
public  static  lista30 Target2Opt( lista30 l30, int pi, int pj) {

  lista30 lTarget = null;
  try {

    lTarget = new lista30( l30);//std-cp

    swap2Opt( lTarget, pi, pj);//Swap2Opt

    if (( new controlPrefs30().modalCriterion()))
      lTarget.ini( index30.unMedida( l30.ParMedida()), listStat30.moda( l30));
      else lTarget.ini( );

    checkList30.chk( lTarget, "Target2Opt");
  }//try
  catch (Exception e) {
    System.out.println( ERROR + "Target2Opt.");
    frame30.exit( 0, e);
  }//catch

  return lTarget;
}//Target2Opt

}//class target30
